#    Copyright (c) 2013 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import random
import string
import time

from yaql.language import specs
from yaql.language import yaqltypes

from muranodashboard.catalog import forms as catalog_forms
from muranodashboard.dynamic_ui import helpers

from castellan.common import exception as castellan_exception
from castellan.common.objects import opaque_data
from castellan import key_manager
from castellan import options

from keystoneauth1 import identity
from keystoneauth1 import session

from django.conf import settings

from oslo_config import cfg
from oslo_context import context as _oslo_context
from oslo_log import log as logging

LOG = logging.getLogger(__name__)


@specs.parameter('times', int)
def _repeat(context, template, times):
    for i in range(times):
        context['index'] = i + 1
        yield helpers.evaluate(template, context)


_random_string_counter = None


@specs.parameter('pattern', yaqltypes.String())
@specs.parameter('number', int)
def _generate_hostname(pattern, number):
    """Generates hostname based on pattern

    Replaces '#' char in pattern with supplied number, if no pattern is
    supplied generates short and unique name for the host.

    :param pattern: hostname pattern
    :param number: number to replace with in pattern
    :return: hostname
    """
    global _random_string_counter

    if pattern:
        # NOTE(kzaitsev) works both for unicode and simple strings in py2
        # and works as expected in py3
        return pattern.replace('#', str(number))

    counter = _random_string_counter or 1
    # generate first 5 random chars
    prefix = ''.join(random.choice(string.ascii_lowercase) for _ in range(5))
    # convert timestamp to higher base to shorten hostname string
    # (up to 8 chars)
    timestamp = helpers.int2base(int(time.time() * 1000), 36)[:8]
    # third part of random name up to 2 chars
    # (1295 is last 2-digit number in base-36, 1296 is first 3-digit number)
    suffix = helpers.int2base(counter, 36)
    _random_string_counter = (counter + 1) % 1296
    return prefix + timestamp + suffix


def _name(context):
    name = context.get_data[
        catalog_forms.WF_MANAGEMENT_NAME]['application_name']
    return name


@specs.parameter('template_name', yaqltypes.String())
@specs.parameter('parameter_name', yaqltypes.String(nullable=True))
@specs.parameter('id_only', yaqltypes.PythonType(bool, nullable=True))
def _ref(context, template_name, parameter_name=None, id_only=None):
    service = context['?service']
    data = None
    if not parameter_name:
        parameter_name = template_name
    # add special symbol to avoid collisions with regular parameters
    # and prevent it from overwriting '?service' context variable
    parameter_name = '#' + parameter_name
    if parameter_name in service.parameters:
        data = service.parameters[parameter_name]
    elif template_name in service.templates:
        data = helpers.evaluate(service.templates[template_name], context)
        service.parameters[parameter_name] = data
    if not isinstance(data, dict):
        return None
    if not isinstance(data.get('?', {}).get('id'), helpers.ObjectID):
        data.setdefault('?', {})['id'] = helpers.ObjectID()
        if id_only is None:
            id_only = False
    elif id_only is None:
        id_only = True

    if id_only:
        return data['?']['id']
    else:
        return data


@specs.parameter('data', yaqltypes.String())
def _encrypt_data(context, data):
    try:
        # TODO(pbourke): move auth construction into common area if it ends up
        # been required in other areas
        auth = identity.V3Password(
            auth_url=settings.KEY_MANAGER['auth_url'],
            username=settings.KEY_MANAGER['username'],
            user_domain_name=settings.KEY_MANAGER['user_domain_name'],
            password=settings.KEY_MANAGER['password'],
            project_name=settings.KEY_MANAGER['project_name'],
            project_domain_name=settings.KEY_MANAGER['project_domain_name']
        )
    except (KeyError, AttributeError) as e:
        LOG.exception(e)
        msg = ('Could not find valid key manager credentials in the '
               'murano-dashboard config. encryptData yaql function not '
               'available')
        raise castellan_exception.KeyManagerError(message_arg=msg)
    sess = session.Session(auth=auth)
    auth_context = _oslo_context.RequestContext(
        auth_token=auth.get_token(sess), tenant=auth.get_project_id(sess))
    options.set_defaults(cfg.CONF,
                         auth_endpoint=settings.KEY_MANAGER['auth_url'])
    options.set_defaults(cfg.CONF,
                         barbican_endpoint_type='internal')

    manager = key_manager.API()
    try:
        # TODO(pbourke): while we feel opaque data should cover the most common
        # use case, we may want to add support for other secret types in the
        # future (see https://goo.gl/tZhfqe)
        stored_key_id = manager.store(auth_context,
                                      opaque_data.OpaqueData(data))
    except castellan_exception.KeyManagerError as e:
        LOG.exception(e)
        raise
    return stored_key_id


def register(context):
    context.register_function(_repeat, 'repeat')
    context.register_function(_generate_hostname, 'generateHostname')
    context.register_function(_name, 'name')
    context.register_function(_ref, 'ref')
    context.register_function(_encrypt_data, 'encryptData')



#    Copyright (c) 2013 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.template import defaultfilters
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django.utils.translation import ngettext_lazy
from horizon import exceptions
from horizon import messages
from horizon import tables
from horizon.utils import filters
from openstack_dashboard import policy
from oslo_log import log as logging

from muranoclient.common import exceptions as exc
from muranodashboard import api
from muranodashboard.common import utils as md_utils

LOG = logging.getLogger(__name__)


class ImportBundle(tables.LinkAction):
    name = 'import_bundle'
    verbose_name = _('Import Bundle')
    url = 'horizon:app-catalog:packages:import_bundle'
    classes = ('ajax-modal',)
    icon = "plus"
    policy_rules = (("murano", "upload_package"),)


class ImportPackage(tables.LinkAction):
    name = 'upload_package'
    verbose_name = _('Import Package')
    url = 'horizon:app-catalog:packages:upload'
    classes = ('ajax-modal',)
    icon = "plus"
    policy_rules = (("murano", "upload_package"),)

    def allowed(self, request, package):
        _allowed = False
        with api.handled_exceptions(request):
            client = api.muranoclient(request)
            if client.categories.list():
                _allowed = True
        return _allowed


class PackagesFilterAction(tables.FilterAction):
    name = "filter_packages"
    filter_type = "server"
    filter_choices = (('search', _("KeyWord"), True),
                      ('type', _("Type"), True),
                      ('name', _("Name"), True))


class DownloadPackage(tables.LinkAction):
    name = 'download_package'
    verbose_name = _('Download Package')
    policy_rules = (("murano", "download_package"),)
    url = 'horizon:app-catalog:packages:download'

    def allowed(self, request, package):
        return True

    def get_link_url(self, app):
        app_name = defaultfilters.slugify(app.name)
        return reverse(self.url, args=(app_name, app.id))


class ToggleEnabled(tables.BatchAction):
    name = 'toggle_enabled'
    verbose_name = _("Toggle Enabled")
    icon = "toggle-on"
    policy_rules = (("murano", "modify_package"),)

    @staticmethod
    def action_present(count):
        return ngettext_lazy(
            u"Toggle Active",
            u"Toggle Active",
            count
        )

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            u"Toggled Active",
            u"Toggled Active",
            count
        )

    def action(self, request, obj_id):
        try:
            api.muranoclient(request).packages.toggle_active(obj_id)
            LOG.debug('Toggle Active for package {0}.'.format(obj_id))
        except exc.HTTPForbidden:
            msg = _("You are not allowed to perform this operation")
            LOG.exception(msg)
            messages.error(request, msg)
            exceptions.handle(
                request,
                msg,
                redirect=reverse('horizon:app-catalog:packages:index'))


class TogglePublicEnabled(tables.BatchAction):
    name = 'toggle_public_enabled'
    verbose_name = _("Toggle Public")
    icon = "share-alt"
    policy_rules = (("murano", "publicize_package"),)

    @staticmethod
    def action_present(count):
        return ngettext_lazy(
            u"Toggle Public",
            u"Toggle Public",
            count
        )

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            u"Toggled Public",
            u"Toggled Public",
            count
        )

    def action(self, request, obj_id):
        try:
            api.muranoclient(request).packages.toggle_public(obj_id)
            LOG.debug('Toggle Public for package {0}.'.format(obj_id))
        except exc.HTTPForbidden:
            msg = _("You are not allowed to perform this operation")
            LOG.exception(msg)
            messages.error(request, msg)
            exceptions.handle(
                request,
                msg,
                redirect=reverse('horizon:app-catalog:packages:index'))
        except exc.HTTPConflict:
            msg = _('Package or Class with the same name is already made '
                    'public')
            LOG.exception(msg)
            messages.error(request, msg)
            exceptions.handle(
                request,
                msg,
                redirect=reverse('horizon:app-catalog:packages:index'))


class DeletePackage(policy.PolicyTargetMixin, tables.DeleteAction):
    name = 'delete_package'
    policy_rules = (("murano", "delete_package"),)

    @staticmethod
    def action_present(count):
        return ngettext_lazy(
            u"Delete Package",
            u"Delete Packages",
            count
        )

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            u"Deleted Package",
            u"Deleted Packages",
            count
        )

    def delete(self, request, obj_id):
        try:
            api.muranoclient(request).packages.delete(obj_id)
        except exc.HTTPNotFound:
            msg = _("Package with id {0} is not found").format(obj_id)
            LOG.exception(msg)
            exceptions.handle(
                self.request,
                msg,
                redirect=reverse('horizon:app-catalog:packages:index'))
        except exc.HTTPForbidden:
            msg = _("You are not allowed to delete this package")
            LOG.exception(msg)
            exceptions.handle(
                request, msg,
                redirect=reverse('horizon:app-catalog:packages:index'))
        except Exception:
            LOG.exception(_('Unable to delete package in murano-api server'))
            url = reverse('horizon:app-catalog:packages:index')
            exceptions.handle(request,
                              _('Unable to remove package.'),
                              redirect=url)


class ModifyPackage(tables.LinkAction):
    name = 'modify_package'
    verbose_name = _('Modify Package')
    url = 'horizon:app-catalog:packages:modify'
    classes = ('ajax-modal',)
    icon = "edit"
    policy_rules = (("murano", "modify_package"),)

    def allowed(self, request, package):
        return True


class PackageDefinitionsTable(tables.DataTable):
    name = md_utils.Column(
        'name',
        link="horizon:app-catalog:packages:detail",
        verbose_name=_('Package Name'))
    tenant_name = tables.Column('tenant_name', verbose_name=_('Tenant Name'))
    enabled = tables.Column('enabled', verbose_name=_('Active'))
    is_public = tables.Column('is_public', verbose_name=_('Public'))
    type = tables.Column('type', verbose_name=_('Type'))
    version = tables.Column(lambda obj: getattr(obj, 'version', None),
                            verbose_name=_('Version'))
    created_time = tables.Column('created',
                                 verbose_name=_('Created'),
                                 filters=(filters.parse_isotime,))
    updated_time = tables.Column('updated',
                                 verbose_name=_('Updated'),
                                 filters=(filters.parse_isotime,))

    def get_prev_pagination_string(self):
        pagination_string = super(
            PackageDefinitionsTable, self).get_prev_pagination_string()
        return pagination_string + "&sort_dir=desc"

    class Meta(object):
        name = 'packages'
        prev_pagination_param = 'marker'
        verbose_name = _('Packages')
        table_actions_menu = (ToggleEnabled,
                              TogglePublicEnabled)
        table_actions = (PackagesFilterAction,
                         ImportPackage,
                         ImportBundle,
                         DeletePackage)

        row_actions = (ModifyPackage,
                       DownloadPackage,
                       ToggleEnabled,
                       TogglePublicEnabled,
                       DeletePackage)



#    Copyright (c) 2016 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

try:
    import cPickle as pickle
except ImportError:
    import pickle
from unittest import mock

import unittest
import yaql

from muranodashboard.common import utils
from muranodashboard.dynamic_ui import yaql_expression


class TestUtils(unittest.TestCase):

    def test_parse_api_error(self):
        test_html = '<html><body><h1>Foo Header</h1>Foo Error </body></html>'
        self.assertEqual('Foo Error', utils.parse_api_error(test_html))

    def test_parse_api_error_without_body(self):
        test_html = '<html></html>'
        self.assertIsNone(utils.parse_api_error(test_html))


class TestCustomPickler(unittest.TestCase):

    def setUp(self):
        super(TestCustomPickler, self).setUp()
        self.custom_pickler = utils.CustomPickler(mock.Mock())
        self.assertTrue(hasattr(self.custom_pickler.dump, '__call__'))
        self.assertTrue(hasattr(self.custom_pickler.clear_memo, '__call__'))

    def test_persistent_id(self):
        yaql_obj = mock.Mock(spec=yaql.factory.YaqlEngine)
        self.assertEqual('filtered:YaqlEngine',
                         self.custom_pickler.persistent_id(yaql_obj))

    def test_persistent_id_with_wrong_obj_type(self):
        self.assertIsNone(self.custom_pickler.persistent_id(None))


class TestCustomUnpickler(unittest.TestCase):

    def setUp(self):
        super(TestCustomUnpickler, self).setUp()
        self.custom_unpickler = utils.CustomUnpickler(mock.Mock())
        self.assertTrue(hasattr(self.custom_unpickler.load, '__call__'))
        if 'noload' in dir(pickle.Unpickler):
            self.assertTrue(hasattr(self.custom_unpickler.noload, '__call__'))

    def test_persistent_load(self):
        result = self.custom_unpickler.persistent_load('filtered:YaqlEngine')
        self.assertEqual(yaql_expression.YAQL, result)

    def test_persistent_load_with_wrong_obj_type(self):
        with self.assertRaises(pickle.UnpicklingError) as cm:
            self.custom_unpickler.persistent_load(None)
        e = cm.exception
        self.assertEqual('Invalid persistent id', str(e))



# Copyright (c) 2016 AT&T Corp
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from django import http
from unittest import mock

from muranodashboard.api.rest import environments

from openstack_dashboard.test import helpers

PARAM_MAPPING = {
    'None': None,
    'True': True,
    'False': False
}


@mock.patch.object(environments, 'api')
@mock.patch.object(environments, 'env_api')
class TestComponentsMetadataAPI(helpers.RestAPITestCase):

    def setUp(self):
        super(TestComponentsMetadataAPI, self).setUp()

        self.request = mock.Mock(body='{"foo": "bar"}', DATA={"foo"})
        self.request.GET = dict(PARAM_MAPPING)
        self.components_metadata = environments.ComponentsMetadata()

        self.addCleanup(mock.patch.stopall)

    def test_get(self, mock_env_api, mock_api):
        mock_sess = mock.Mock()
        mock_component = mock.Mock()
        test_response = http.HttpResponse(
            "Foobar metadata response.", content_type="text/plain")
        mock_component.to_dict.return_value = {
            '?': {
                'metadata': test_response
            }
        }
        mock_env_api.Session.get_or_create_or_delete.return_value = mock_sess
        mock_api.muranoclient().services.get.return_value = mock_component

        response = self.components_metadata.get(
            self.request, 'foo_env', 'foo_component')

        self.assertEqual(test_response, response)
        self.assertEqual(b"Foobar metadata response.", response.content)
        mock_env_api.Session.get_or_create_or_delete.assert_called_once_with(
            self.request, 'foo_env')
        mock_api.muranoclient().services.get.assert_called_once_with(
            'foo_env', '/foo_component', mock_sess)

    def test_get_empty_response(self, mock_env_api, mock_api):
        mock_env_api.Session.get_or_create_or_delete.return_value = mock.Mock()
        mock_api.muranoclient().services.get.return_value = None
        result = self.components_metadata.get(
            self.request, None, 'foo_component')
        self.assertEqual(200, result.status_code)
        self.assertEqual(b'{}', result.content)

    def test_post_updated(self, mock_env_api, mock_api):
        mock_sess = mock.Mock()
        mock_env_api.Session.get_or_create_or_delete.return_value = mock_sess

        self.request.body = '{"updated": true}'
        self.components_metadata.post(self.request, 'foo_env', 'foo_component')

        mock_env_api.Session.get_or_create_or_delete.assert_called_once_with(
            self.request, 'foo_env')
        mock_api.muranoclient.assert_called_once_with(self.request)
        mock_api.muranoclient().services.put.assert_called_once_with(
            'foo_env', '/foo_component/%3F/metadata', True, mock_sess)


@mock.patch.object(environments, 'api')
@mock.patch.object(environments, 'env_api')
class TestEnvironmentsMetadataApi(helpers.RestAPITestCase):

    def setUp(self):
        super(TestEnvironmentsMetadataApi, self).setUp()

        self.request = mock.Mock(body='{"foo": "bar"}', DATA={"foo"})
        self.request.GET = dict(PARAM_MAPPING)
        self.envs_metadata = environments.EnvironmentsMetadata()

        self.addCleanup(mock.patch.stopall)

    def test_get(self, mock_env_api, mock_api):
        mock_sess = mock.Mock()
        http_response = http.HttpResponse(
            "Foobar metadata response.", content_type="text/plain")
        test_response = {
            '?': {
                'metadata': http_response
            }
        }
        mock_env_api.Session.get_or_create_or_delete.return_value = mock_sess
        mock_api.muranoclient().environments.get_model.return_value =\
            test_response

        response = self.envs_metadata.get(self.request, 'foo_env')
        self.assertEqual(http_response, response)
        self.assertEqual(b"Foobar metadata response.", response.content)

        mock_env_api.Session.get_or_create_or_delete.assert_called_once_with(
            self.request, 'foo_env')
        mock_api.muranoclient().environments.get_model.assert_called_once_with(
            'foo_env', '/', mock_sess)

    def test_get_empty_response(self, mock_env_api, mock_api):
        mock_env_api.Session.get_or_create_or_delete.return_value = mock.Mock()
        mock_api.muranoclient().environments.get_model.return_value = None

        result = self.envs_metadata.get(self.request, 'foo_env')
        self.assertEqual(200, result.status_code)
        self.assertEqual(b'{}', result.content)

    def test_post(self, mock_env_api, mock_api):
        mock_sess = mock.Mock()
        mock_env_api.Session.get_or_create_or_delete.return_value = mock_sess

        self.request.body = '{"updated": true}'
        self.envs_metadata.post(self.request, 'foo_env')

        expected_patch = {
            "op": "replace",
            "path": "/?/metadata",
            "value": True
        }
        mock_api.muranoclient().environments.update_model.\
            assert_called_once_with('foo_env', [expected_patch], mock_sess)



#    Copyright (c) 2013 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import itertools

from django.urls import reverse
from django.urls import reverse_lazy
from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon.forms import views
from horizon import tables as horizon_tables
from horizon.utils import functions as utils
from openstack_dashboard.api import glance

from muranodashboard.images import forms
from muranodashboard.images import tables


class MarkedImagesView(horizon_tables.DataTableView):
    table_class = tables.MarkedImagesTable
    template_name = 'images/index.html'
    page_title = _("Marked Images")

    def has_prev_data(self, table):
        return self._prev

    def has_more_data(self, table):
        return self._more

    def get_data(self):
        prev_marker = self.request.GET.get(
            tables.MarkedImagesTable._meta.prev_pagination_param, None)

        if prev_marker is not None:
            sort_dir = 'asc'
            marker = prev_marker
        else:
            sort_dir = 'desc'
            marker = self.request.GET.get(
                tables.MarkedImagesTable._meta.pagination_param, None)

        page_size = utils.get_page_size(self.request)

        request_size = page_size + 1
        kwargs = {'filters': {}}
        if marker:
            kwargs['marker'] = marker
        kwargs['sort_dir'] = sort_dir
        images = []
        self._prev = False
        self._more = False

        glance_v2_client = glance.glanceclient(self.request)

        try:
            images_iter = glance_v2_client.images.list(
                **kwargs)
        except Exception:
            msg = _('Unable to retrieve list of images')
            uri = reverse('horizon:app-catalog:catalog:index')

            exceptions.handle(self.request, msg, redirect=uri)

        marked_images_iter = forms.filter_murano_images(
            images_iter,
            request=self.request)
        images = list(itertools.islice(marked_images_iter, request_size))
        # first and middle page condition
        if len(images) > page_size:
            images.pop(-1)
            self._more = True
            # middle page condition
            if marker is not None:
                self._prev = True
        # first page condition when reached via prev back
        elif sort_dir == 'asc' and marker is not None:
            self._more = True
        # last page condition
        elif marker is not None:
            self._prev = True
        if prev_marker is not None:
            images.reverse()
        return images


class MarkImageView(views.ModalFormView):
    form_class = forms.MarkImageForm
    form_id = 'mark_murano_image_form'
    modal_header = _('Add Murano Metadata')
    template_name = 'images/mark.html'
    context_object_name = 'image'
    page_title = _("Update Image")
    success_url = reverse_lazy('horizon:app-catalog:images:index')
    submit_label = _('Mark Image')
    submit_url = reverse_lazy('horizon:app-catalog:images:mark_image')



#    Copyright (c) 2013 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import tempfile

from django.conf import settings
from django.utils.translation import gettext_lazy as _


# ---- Metadata Consts ---- #
CHUNK_SIZE = 1 << 20  # 1MB
ARCHIVE_PKG_NAME = 'archive.tar.gz'
CACHE_DIR = getattr(settings, 'METADATA_CACHE_DIR',
                    os.path.join(tempfile.gettempdir(),
                                 'muranodashboard-cache'))

CACHE_REFRESH_SECONDS_INTERVAL = 5

DASHBOARD_ATTRS_KEY = '_26411a1861294160833743e45d0eaad9'

# ---- Forms Consts ---- #
STATUS_ID_READY = 'ready'
STATUS_ID_PENDING = 'pending'
STATUS_ID_DEPLOYING = 'deploying'
STATUS_ID_DELETING = 'deleting'
STATUS_ID_DELETE_FAILURE = 'delete failure'
STATUS_ID_DEPLOY_FAILURE = 'deploy failure'
STATUS_ID_NEW = 'new'

NO_ACTION_ALLOWED_STATUSES = (STATUS_ID_DEPLOYING,
                              STATUS_ID_DELETING)

DEP_STATUS_ID_RUNNING = 'running'
DEP_STATUS_ID_RUNNING_W_ERRORS = 'running_w_errors'
DEP_STATUS_ID_RUNNING_W_WARNINGS = 'running_w_warnings'
DEP_STATUS_ID_COMPLETED_W_ERRORS = 'completed_w_errors'
DEP_STATUS_ID_COMPLETED_W_WARNINGS = 'completed_w_warnings'
DEP_STATUS_ID_SUCCESS = 'success'

# A tuple of tuples representing the possible data values for the
# status column and their associated boolean equivalent. Positive
# states should equate to ``True``, negative states should equate
# to ``False``, and indeterminate states should be ``None``.
# When value is None progress bar will be displayed.

STATUS_CHOICES = (
    (None, True),
    (STATUS_ID_READY, True),
    (STATUS_ID_PENDING, True),
    (STATUS_ID_DEPLOYING, None),
    (STATUS_ID_DELETING, None),
    (STATUS_ID_NEW, True),
    (STATUS_ID_DELETE_FAILURE, False),
    (STATUS_ID_DEPLOY_FAILURE, False),
)

DEPLOYMENT_STATUS_CHOICES = (
    (None, True),
    (DEP_STATUS_ID_RUNNING, True),
    (DEP_STATUS_ID_SUCCESS, True),
    (DEP_STATUS_ID_RUNNING_W_ERRORS, False),
    (DEP_STATUS_ID_RUNNING_W_WARNINGS, False),
    (DEP_STATUS_ID_COMPLETED_W_WARNINGS, False),
    (DEP_STATUS_ID_COMPLETED_W_ERRORS, False),
)

STATUS_DISPLAY_CHOICES = (
    (STATUS_ID_READY, _('Ready')),
    (STATUS_ID_DEPLOYING, _('Deploying')),
    (STATUS_ID_DELETING, _('Deleting')),
    (STATUS_ID_PENDING, _('Ready to deploy')),
    (STATUS_ID_NEW, _('Ready to configure')),
    (STATUS_ID_DELETE_FAILURE, _('Delete failure')),
    (STATUS_ID_DEPLOY_FAILURE, _('Deploy failure')),
    ('', _('Ready to configure')),
)

DEPLOYMENT_STATUS_DISPLAY_CHOICES = (
    (DEP_STATUS_ID_COMPLETED_W_ERRORS, _('Failed')),
    (DEP_STATUS_ID_COMPLETED_W_WARNINGS, _('Completed with warnings')),
    (DEP_STATUS_ID_RUNNING, _('Running')),
    (DEP_STATUS_ID_RUNNING_W_ERRORS, _('Running with errors')),
    (DEP_STATUS_ID_RUNNING_W_WARNINGS, _('Running with warnings')),
    (DEP_STATUS_ID_SUCCESS, _('Successful')),
    ('', _('Unknown')),
)



# Copyright (c) 2016 AT&T Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import unittest
from unittest import mock

from django.utils.translation import gettext_lazy as _

from muranoclient.common import exceptions as exc
from muranodashboard.environments import forms as env_forms


class TestCreateEnvForm(unittest.TestCase):
    def setUp(self):
        super(TestCreateEnvForm, self).setUp()
        self.mock_request = mock.MagicMock()
        self.data = {
            'name': 'test',
            'net_config': 'Nova'
        }

    @mock.patch.object(env_forms, 'ast')
    @mock.patch.object(env_forms, 'api')
    @mock.patch('muranodashboard.common.net.get_available_networks')
    def test_handle(self, mock_net, mock_api, mock_ast):
        mock_net.return_value = None
        test_env_form = env_forms.CreateEnvironmentForm(self.mock_request)
        self.assertEqual([((None, None), _('Unavailable'))],
                         test_env_form.fields['net_config'].choices)
        self.assertEqual(env_forms.net.NN_HELP,
                         test_env_form.fields['net_config'].help_text)

        self.assertTrue(test_env_form.handle(self.mock_request, self.data))
        mock_api.environment_create.assert_called_once_with(self.mock_request,
                                                            self.data)
        mock_ast.literal_eval.assert_called_once_with('Nova')

    @mock.patch.object(env_forms, 'LOG')
    @mock.patch.object(env_forms, 'ast')
    @mock.patch.object(env_forms, 'api')
    @mock.patch('muranodashboard.common.net.get_available_networks')
    def test_handle_error(self, mock_net, mock_api, mock_ast, mock_log):
        mock_net.return_value = None
        test_env_form = env_forms.CreateEnvironmentForm(self.mock_request)
        self.assertEqual([((None, None), _('Unavailable'))],
                         test_env_form.fields['net_config'].choices)
        self.assertEqual(env_forms.net.NN_HELP,
                         test_env_form.fields['net_config'].help_text)

        mock_api.environment_create.side_effect = exc.HTTPConflict
        msg = _('Environment with specified name already exists')
        self.assertRaises(exc.HTTPConflict, test_env_form.handle,
                          self.mock_request, self.data)
        mock_ast.literal_eval.assert_called_once_with('Nova')
        mock_log.exception.assert_called_once_with(msg)



# Copyright (c) 2016 AT&T Corp
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from django.core.files import storage
from django import http
from django.utils.translation import gettext_lazy as _
from unittest import mock


from horizon import exceptions as horizon_exceptions
from muranoclient.common import exceptions as exc

from muranodashboard.packages import consts as packages_consts
from muranodashboard.packages import forms
from muranodashboard.packages import tables
from muranodashboard.packages import views

from openstack_dashboard.test import helpers


class TestPackageView(helpers.APITestCase):

    def setUp(self):
        super(TestPackageView, self).setUp()

        fake_response = {'status_code': 200}
        self.mock_request = mock.Mock(return_value=fake_response)

        self.addCleanup(mock.patch.stopall)

    @mock.patch('muranodashboard.packages.views.api.muranoclient')
    def test_download_package(self, mock_client):
        mock_client().packages.download.return_value = {}

        response =\
            views.download_packge(self.mock_request, 'test_app_name', None)
        expected_response = {
            'Content-Type': 'application/octet-stream',
            'Content-Disposition': 'filename=test_app_name.zip'
        }

        self.assertIsInstance(response, http.HttpResponse)
        for key, val in expected_response.items():
            key = key.lower()
            self.assertTrue(response.has_header(key))
            self.assertEqual(val, response[key])

    @mock.patch.object(views, 'exceptions')
    @mock.patch.object(views, 'reverse')
    @mock.patch.object(views, 'LOG')
    @mock.patch('muranodashboard.packages.views.api.muranoclient')
    def test_download_package_except_http_exception(self, mock_client,
                                                    mock_log, mock_reverse,
                                                    mock_exc):
        mock_client().packages.download.side_effect = exc.HTTPException
        mock_reverse.return_value = 'test_redirect'

        views.download_packge(self.mock_request, 'test_app_name', None)

        mock_log.exception.assert_called_once_with(
            'Something went wrong during package downloading')
        mock_reverse.assert_called_once_with(
            'horizon:app-catalog:packages:index')
        mock_exc.handle.assert_called_once_with(
            self.mock_request, 'Unable to download package.',
            redirect='test_redirect')

    def test_is_app(self):
        mock_wizard = mock.Mock()
        mock_step_data = mock.MagicMock()
        mock_step_data.__getitem__().type = 'Application'
        mock_wizard.storage.get_step_data.return_value = mock_step_data
        self.assertTrue(views.is_app(mock_wizard))

        mock_step_data.__getitem__().type = 'Non-Application'
        mock_wizard.storage.get_step_data.return_value = mock_step_data
        self.assertFalse(views.is_app(mock_wizard))


class TestDetailView(helpers.APITestCase):

    def setUp(self):
        super(TestDetailView, self).setUp()

        self.detail_view = views.DetailView()
        fake_response = {'status_code': 200}
        self.mock_request = mock.Mock(return_value=fake_response)
        self.detail_view.request = self.mock_request
        self.detail_view.kwargs = {'app_id': 'foo'}

        self.assertEqual('packages/detail.html',
                         self.detail_view.template_name)
        self.assertEqual('{{ app.name }}', self.detail_view.page_title)

        self.addCleanup(mock.patch.stopall)

    @mock.patch('muranodashboard.packages.views.api.muranoclient')
    def test_get_context_data(self, mock_client):
        mock_client().packages.get.return_value = 'test_app'

        context = self.detail_view.get_context_data()
        self.assertIn('app', context)
        self.assertIn('view', context)
        self.assertIsInstance(context['view'], views.DetailView)
        self.assertEqual('test_app', context['app'])

    @mock.patch.object(views, 'exceptions')
    @mock.patch.object(views, 'reverse')
    @mock.patch('muranodashboard.packages.views.api.muranoclient')
    def test_get_context_data_except_exception(self, mock_client, mock_reverse,
                                               mock_exc):
        mock_client().packages.get.side_effect = Exception
        mock_reverse.return_value = 'test_redirect'

        self.detail_view.get_context_data()

        mock_exc.handle.assert_called_once_with(
            self.mock_request, 'Unable to retrieve package details.',
            redirect='test_redirect')


class TestModifyPackageView(helpers.RestAPITestCase):

    def setUp(self):
        super(TestModifyPackageView, self).setUp()

        self.modify_pkg_view = views.ModifyPackageView()
        fake_response = {'status_code': 200}
        self.mock_request = mock.Mock(return_value=fake_response, META=[])
        self.modify_pkg_view.request = self.mock_request
        self.modify_pkg_view.kwargs = {'app_id': 'foo'}

        self.assertEqual(forms.ModifyPackageForm,
                         self.modify_pkg_view.form_class)
        self.assertEqual('packages/modify_package.html',
                         self.modify_pkg_view.template_name)
        self.assertEqual('Modify Package', self.modify_pkg_view.page_title)

        self.addCleanup(mock.patch.stopall)

    @mock.patch('muranodashboard.packages.views.api.muranoclient')
    def test_get_initial(self, mock_client):
        mock_package = mock.Mock()
        mock_client().packages.get.return_value = mock_package
        expected_result = {
            'package': mock_package,
            'app_id': 'foo'
        }

        result = self.modify_pkg_view.get_initial()

        for key, val in expected_result.items():
            self.assertEqual(val, result[key])

    def test_get_context_data(self):
        mock_form = mock.Mock(return_value=type(
            'FakeForm', (object, ), {'initial': {
                'package': type(
                    'FakeFormInner', (object, ), {'type': 'test_type'}
                )
            }}
        ))
        self.modify_pkg_view.get_form = mock_form
        expected_context = {
            'app_id': 'foo',
            'type': 'test_type',
            'form': mock_form
        }

        context = self.modify_pkg_view.get_context_data(form=mock_form)

        self.assertIn('view', context)
        self.assertIsInstance(context['view'], views.ModifyPackageView)

        for key, val in expected_context.items():
            self.assertIn(key, context)
            self.assertEqual(val, context[key])
        self.modify_pkg_view.get_form.assert_called_once_with()


class TestImportPackageWizard(helpers.APITestCase):

    def setUp(self):
        super(TestImportPackageWizard, self).setUp()

        fake_response = {'status_code': 200}
        self.mock_request = mock.MagicMock(return_value=fake_response, META=[])

        self.import_pkg_wizard = views.ImportPackageWizard()
        self.import_pkg_wizard.request = self.mock_request

        all_cleaned_data = {
            'enabled': False,
            'is_public': True,
            'package': 'test_package',
            'import_type': 'test_import_type',
            'url': 'test_url',
            'repo_version': 'test_repo_version',
            'repo_name': 'test_repo_name',
            'tags': 'foo,bar,baz,qux'
        }
        self.import_pkg_wizard.get_all_cleaned_data = lambda: all_cleaned_data
        self.import_pkg_wizard.steps = mock.Mock(current='upload')

        self.assertIsInstance(self.import_pkg_wizard.file_storage,
                              storage.FileSystemStorage)
        self.assertEqual('packages/upload.html',
                         self.import_pkg_wizard.template_name)
        self.assertEqual({'add_category': views.is_app},
                         self.import_pkg_wizard.condition_dict)
        self.assertEqual('Import Package',
                         self.import_pkg_wizard.page_title)

        self.addCleanup(mock.patch.stopall)

    def test_get_form_initial(self):
        self.mock_request.GET = {
            'url': 'test_url',
            'repo_name': 'test_repo_name',
            'repo_version': 'test_repo_version',
            'import_type': 'test_import_type'
        }
        expected_dict = {
            'foo': 'bar',
            'url': 'test_url',
            'repo_name': 'test_repo_name',
            'repo_version': 'test_repo_version',
            'import_type': 'test_import_type'
        }

        self.import_pkg_wizard.initial_dict = {'upload': {'foo': 'bar'}}
        initial_dict = self.import_pkg_wizard.get_form_initial('upload')

        for key, val in expected_dict.items():
            self.assertIn(key, initial_dict)
            self.assertEqual(val, initial_dict[key])

    def test_get_context_data(self):
        mock_form = mock.Mock()

        self.import_pkg_wizard.storage = mock.Mock(
            extra_data={'extra': 'data'})
        self.import_pkg_wizard.prefix = 'test_prefix'

        expected_result = {
            'form': mock_form,
            'modal_backdrop': 'static',
            'extra': 'data',
            'murano_repo_url': 'http://apps.openstack.org',
            'wizard': {
                'steps': self.import_pkg_wizard.steps,
                'form': mock_form
            }
        }

        result = self.import_pkg_wizard.get_context_data(form=mock_form)

        self.assertIn('view', result)
        self.assertIsInstance(result['view'], views.ImportPackageWizard)

        for key, val in expected_result.items():
            if isinstance(val, dict):
                for key_, val_ in val.items():
                    self.assertEqual(val_, val[key_])
            else:
                self.assertEqual(val, result[key])

    @mock.patch.object(views, 'exceptions')
    @mock.patch.object(views, 'reverse')
    @mock.patch.object(views, 'glance')
    @mock.patch.object(views, 'api')
    def test_done(self, mock_api, mock_glance, mock_reverse, mock_exc):
        mock_storage = mock.MagicMock()
        mock_storage.get_step_data().__getitem__.return_value =\
            mock.Mock(id='test_package_id')
        mock_storage.get_step_data().get.side_effect = [
            [mock.Mock(id='test_dep_pkg_id', fully_qualified_name='fqn')],
            [{'id': 'test_image_id'}]
        ]

        self.import_pkg_wizard.storage = mock_storage
        self.import_pkg_wizard.form_list = {}

        self.import_pkg_wizard.done({})

        expected_api_mock_calls = [
            mock.call('test_dep_pkg_id',
                      {'enabled': False, 'is_public': True}),
            mock.call('test_package_id',
                      {'enabled': False, 'is_public': True,
                       'tags': ['foo', 'bar', 'baz', 'qux']}),
        ]

        mock_api.muranoclient().packages.update.assert_has_calls(
            expected_api_mock_calls)
        mock_reverse.assert_called_once_with(
            'horizon:app-catalog:packages:index')
        mock_glance.glanceclient().images.update.assert_called_once_with(
            mock.ANY, is_public=True)
        mock_storage.get_step_data.assert_any_call('upload')
        mock_storage.get_step_data().get.assert_any_call('dependencies', [])
        mock_storage.get_step_data().get.assert_any_call('images', [])
        mock_exc.handle.assert_not_called()

    @mock.patch.object(views, 'reverse')
    @mock.patch.object(views, 'glance')
    @mock.patch.object(views, 'LOG')
    @mock.patch.object(views, 'messages')
    @mock.patch.object(views, 'api')
    def test_done_except_murano_client_exception(self, mock_api, mock_messages,
                                                 mock_log, *args):
        mock_storage = mock.MagicMock()
        mock_storage.get_step_data().__getitem__.return_value =\
            mock.Mock(id='test_package_id')
        mock_storage.get_step_data().get.side_effect = [
            [mock.Mock(id='test_dep_pkg_id', fully_qualified_name='fqn')],
            [{'id': 'test_image_id'}]
        ]
        mock_api.muranoclient().packages.update.side_effect = [
            Exception("murano client error message."), None
        ]

        self.import_pkg_wizard.storage = mock_storage
        self.import_pkg_wizard.form_list = {}

        self.import_pkg_wizard.done({})

        expected_msg = "Couldn't update package {0} parameters. Error: {1}"\
                       .format('fqn', 'murano client error message.')
        mock_log.warning.assert_called_once_with(expected_msg)
        mock_messages.warning.assert_called_once_with(
            self.mock_request, expected_msg)

    @mock.patch.object(views, 'api')
    @mock.patch.object(views, 'reverse')
    @mock.patch.object(views, 'glance')
    @mock.patch.object(views, 'LOG')
    @mock.patch.object(views, 'messages')
    def test_done_except_glance_init_exception(self, mock_messages, mock_log,
                                               mock_glance, *args):
        """Test null glance client with installed images throws exception."""
        mock_storage = mock.MagicMock()
        mock_storage.get_step_data().__getitem__.return_value =\
            mock.Mock(id='test_package_id')
        mock_storage.get_step_data().get.side_effect = [
            [mock.Mock(id='test_dep_pkg_id', fully_qualified_name='fqn')],
            [{'id': 'foo_image_id', 'name': 'foo_image_name'},
             {'id': 'bar_image_id', 'name': 'bar_image_name'}]
        ]
        mock_glance.glanceclient.return_value = None

        self.import_pkg_wizard.storage = mock_storage
        self.import_pkg_wizard.form_list = {}

        self.import_pkg_wizard.done({})

        expected_msg = "Couldn't initialise glance v1 client, therefore "\
                       "could not make the following images public: {0}"\
                       .format('foo_image_name bar_image_name')
        mock_log.warning.assert_called_once_with(expected_msg)
        mock_messages.warning.assert_called_once_with(
            self.mock_request, expected_msg)

    @mock.patch.object(views, 'api')
    @mock.patch.object(views, 'reverse')
    @mock.patch.object(views, 'glance')
    @mock.patch.object(views, 'LOG')
    @mock.patch.object(views, 'messages')
    def test_done_except_glance_update_exception(self, mock_messages, mock_log,
                                                 mock_glance, *args):
        mock_storage = mock.MagicMock()
        mock_storage.get_step_data().__getitem__.return_value =\
            mock.Mock(id='test_package_id')
        mock_storage.get_step_data().get.side_effect = [
            [mock.Mock(id='test_dep_pkg_id', fully_qualified_name='fqn')],
            [{'id': 'test_image_id', 'name': 'test_image_name'}]
        ]
        glance_exception = Exception("glance client error message")
        mock_glance.glanceclient().images.update.side_effect =\
            glance_exception

        self.import_pkg_wizard.storage = mock_storage
        self.import_pkg_wizard.form_list = {}

        self.import_pkg_wizard.done({})

        expected_msg = "Error {0} occurred while setting image {1}, {2} "\
                       "public".format(glance_exception, "test_image_name",
                                       "test_image_id")
        mock_log.exception.assert_called_once_with(expected_msg)
        mock_messages.error.assert_called_once_with(
            self.mock_request, expected_msg)

    @mock.patch.object(views, 'glance')
    @mock.patch.object(views, 'reverse')
    @mock.patch.object(views, 'LOG')
    @mock.patch.object(views, 'exceptions')
    @mock.patch.object(views, 'api')
    def test_done_except_http_forbidden(self, mock_api, mock_exc, mock_log,
                                        mock_reverse, _):
        mock_storage = mock.MagicMock()
        mock_storage.get_step_data().__getitem__.return_value =\
            mock.Mock(id='test_package_id')
        mock_storage.get_step_data().get.side_effect = [
            [mock.Mock(id='test_dep_pkg_id', fully_qualified_name='fqn')],
            [{'id': 'test_image_id', 'name': 'test_image_name'}]
        ]
        mock_api.muranoclient().packages.update.side_effect = [
            None, exc.HTTPForbidden
        ]
        mock_reverse.return_value = 'test_redirect'

        self.import_pkg_wizard.storage = mock_storage
        self.import_pkg_wizard.form_list = {}

        self.import_pkg_wizard.done({})

        expected_msg = "You are not allowed to change this properties of the "\
                       "package"
        mock_log.exception.assert_called_once_with(expected_msg)
        mock_exc.handle.assert_called_once_with(
            self.mock_request, expected_msg, redirect='test_redirect')

    @mock.patch.object(views, 'glance')
    @mock.patch.object(views, 'reverse')
    @mock.patch.object(views, 'LOG')
    @mock.patch.object(views, 'exceptions')
    @mock.patch.object(views, 'api')
    def test_done_except_http_exception(self, mock_api, mock_exc, mock_log,
                                        mock_reverse, _):
        mock_storage = mock.MagicMock()
        mock_storage.get_step_data().__getitem__.return_value =\
            mock.Mock(id='test_package_id')
        mock_storage.get_step_data().get.side_effect = [
            [mock.Mock(id='test_dep_pkg_id', fully_qualified_name='fqn')],
            [{'id': 'test_image_id', 'name': 'test_image_name'}]
        ]
        mock_api.muranoclient().packages.update.side_effect = [
            None, exc.HTTPException
        ]
        mock_reverse.return_value = 'test_redirect'

        self.import_pkg_wizard.storage = mock_storage
        self.import_pkg_wizard.form_list = {}

        self.import_pkg_wizard.done({})

        expected_msg = 'Modifying package failed'
        mock_log.exception.assert_called_once_with(expected_msg)
        mock_exc.handle.assert_called_once_with(
            self.mock_request, 'Unable to modify package',
            redirect='test_redirect')

    @mock.patch.object(views, 'reverse')
    @mock.patch.object(views, 'exceptions')
    @mock.patch.object(views, 'LOG')
    def test_handle_exception(self, mock_log, mock_exc, mock_reverse):
        mock_exception = mock.Mock()
        mock_exception.details = '{"error": {"message": "test_error_message"}}'
        mock_reverse.return_value = 'test_redirect'

        self.import_pkg_wizard.request = self.mock_request
        self.import_pkg_wizard._handle_exception(mock_exception)

        expected_msg = 'Uploading package failed. {0}'\
                       .format('test_error_message')
        mock_log.exception.assert_called_once_with(expected_msg)
        mock_exc.handle.assert_called_once_with(
            self.mock_request, expected_msg, redirect='test_redirect')
        mock_reverse.assert_called_once_with(
            'horizon:app-catalog:packages:index')

    @mock.patch.object(views, 'json')
    def test_handle_exception_except_value_error(self, mock_json):
        mock_json.loads.side_effect = ValueError('test_error_message')
        original_e = ValueError('original_error_message')
        setattr(original_e, 'details', 'error_details')
        with self.assertRaisesRegex(ValueError, 'original_error_message'):
            self.import_pkg_wizard._handle_exception(original_e)

    @mock.patch.object(views, 'glance')
    @mock.patch.object(views, 'messages')
    @mock.patch.object(views, 'api')
    @mock.patch.object(views, 'muranoclient_utils')
    def test_process_step(self, mock_murano_utils, mock_api, mock_messages, _):
        mock_package = mock.Mock()
        mock_package.manifest = {'FullName': 'foo'}
        mock_original_package = mock.Mock()
        mock_original_package.file.return_value = 'foo_file'
        mock_package.requirements.return_value =\
            {'foo': mock_original_package, 'bar': mock.Mock()}
        mock_murano_utils.ensure_images.return_value = [
            {'id': 1, 'name': 'foo'}, {'id': 2, 'name': 'bar'}
        ]
        mock_murano_utils.Package.from_file.return_value = mock_package

        mock_dependency_package = mock.Mock()
        mock_result_package = mock.Mock(id='result_package_id')
        mock_api.muranoclient().packages.create.side_effect = [
            mock_dependency_package,
            mock_result_package
        ]

        mock_form = mock.Mock()
        mock_form.cleaned_data = {
            'import_type': 'upload',
            'package': mock.Mock(file='test_package_file')
        }
        mock_form.data = {
            'dependencies': ['dep1', 'dep2'],
            'images': ['img1', 'img2']
        }

        step_data = self.import_pkg_wizard.process_step(mock_form)

        for expected_key in ('images', 'dependencies', 'package'):
            self.assertIn(expected_key, step_data)
        self.assertIn({'id': 1, 'name': 'foo'}, step_data['images'])
        self.assertIn({'id': 2, 'name': 'bar'}, step_data['images'])
        self.assertEqual([mock_dependency_package], step_data['dependencies'])
        self.assertEqual(mock_result_package, step_data['package'])
        self.assertEqual(2, mock_murano_utils.ensure_images.call_count)

        mock_murano_utils.Package.from_file.assert_called_once_with(
            'test_package_file')
        mock_api.muranoclient().packages.create.assert_any_call(
            mock.ANY, {'foo': 'foo_file'})
        mock_package.requirements.assert_called_once_with(
            base_url=packages_consts.MURANO_REPO_URL)
        mock_messages.success.assert_any_call(
            self.mock_request, 'Package bar uploaded')
        mock_messages.success.assert_any_call(
            self.mock_request, 'Package foo uploaded')

    @mock.patch.object(views, 'reverse')
    @mock.patch.object(views, 'LOG')
    @mock.patch.object(views, 'muranoclient_utils')
    def test_process_step_except_from_file_exception(
            self, mock_murano_utils, mock_log, mock_reverse):
        mock_reverse.return_value = 'test_redirect'
        mock_form = mock.Mock()
        mock_form.cleaned_data = {
            'import_type': 'upload',
            'package': mock.Mock(file='test_package_file')
        }

        errors = (('(404) test_404_error', "Package creation failed."
                                           "Reason: Can't find Package name "
                                           "from repository."),
                  ('random error message', "Package creation failed.Reason: "
                                           "random error message"))
        for error_message, expected_error_message in errors:
            exception = Exception(error_message)
            exception.message = error_message
            mock_murano_utils.Package.from_file.side_effect = exception

            with self.assertRaisesRegex(horizon_exceptions.Http302,
                                        None):
                self.import_pkg_wizard.process_step(mock_form)

            mock_log.exception.assert_called_once_with(expected_error_message)
            mock_reverse.assert_called_once_with(
                'horizon:app-catalog:packages:index')
            mock_log.exception.reset_mock()
            mock_reverse.reset_mock()

    @mock.patch.object(views, 'exceptions')
    @mock.patch.object(views, 'messages')
    @mock.patch.object(views, 'reverse')
    @mock.patch.object(views, 'LOG')
    @mock.patch.object(views, 'api')
    @mock.patch.object(views, 'muranoclient_utils')
    @mock.patch.object(views, 'glance')
    def test_process_step_except_package_create_exception(
            self, mock_glance, mock_murano_utils, mock_api, mock_log,
            mock_reverse, mock_messages, mock_exc):
        mock_murano_utils.Package.from_file.side_effect = None
        mock_reverse.return_value = 'test_redirect'
        mock_package = mock.Mock()
        mock_package.manifest = {'FullName': 'foo'}
        mock_murano_utils.Package.from_file.return_value = mock_package

        mock_form = mock.Mock()
        mock_form.cleaned_data = {
            'import_type': 'upload',
            'package': mock.Mock(file='test_package_file')
        }
        mock_form.data = {
            'dependencies': [],
            'images': []
        }

        # Test that first occurrence of exception is handled.
        mock_package.requirements.return_value =\
            {'foo': mock.Mock(), 'bar': mock.Mock()}
        expected_error_message = _(
            "Error test_error_message occurred while installing package bar")
        mock_api.muranoclient().packages.create.side_effect = [
            Exception('test_error_message'), mock.Mock(id='test_package_id')
        ]
        self.import_pkg_wizard.process_step(mock_form)
        mock_messages.error.assert_any_call(
            self.mock_request, expected_error_message)
        mock_messages.success.assert_any_call(
            self.mock_request, _('Package foo uploaded'))
        mock_log.exception.assert_called_once_with(expected_error_message)
        mock_log.exception.reset_mock()
        mock_messages.reset_mock()

        # Test that second occurrence of exception is handled.
        mock_package.requirements.return_value =\
            {'foo': mock.Mock(), 'bar': mock.Mock()}
        expected_error_message = 'Uploading package failed. {0}'.format('')
        mock_api.muranoclient().packages.create.side_effect = [
            mock.Mock(id='test_package_id'), Exception
        ]
        self.import_pkg_wizard.process_step(mock_form)
        mock_log.exception.assert_called_once_with(expected_error_message)
        mock_exc.handle.assert_called_once_with(
            self.mock_request, expected_error_message,
            redirect='test_redirect')
        mock_reverse.assert_called_once_with(
            'horizon:app-catalog:packages:index')

    @mock.patch.object(views, 'exceptions')
    @mock.patch.object(views, 'messages')
    @mock.patch.object(views, 'reverse')
    @mock.patch.object(views, 'LOG')
    @mock.patch.object(views, 'api')
    @mock.patch.object(views, 'muranoclient_utils')
    @mock.patch.object(views, 'glance')
    def test_process_step_except_http_conflict(
            self, mock_glance, mock_murano_utils, mock_api, mock_log,
            mock_reverse, mock_messages, mock_exc):
        mock_murano_utils.Package.from_file.side_effect = None
        mock_reverse.return_value = 'test_redirect'

        mock_form = mock.Mock()
        mock_form.cleaned_data = {
            'import_type': 'upload',
            'package': mock.Mock(file='test_package_file')
        }
        mock_form.data = {
            'dependencies': [],
            'images': []
        }

        # Test that first occurrence of HTTPConflict is caught.
        mock_package = mock.Mock()
        mock_package.manifest = {'FullName': 'foo'}
        mock_package.requirements.return_value =\
            {'foo': mock.Mock(), 'bar': mock.Mock()}
        mock_murano_utils.Package.from_file.return_value = mock_package
        expected_error_message = "Package bar already registered."
        mock_api.muranoclient().packages.create.side_effect = [
            exc.HTTPConflict, None
        ]
        self.import_pkg_wizard.process_step(mock_form)
        mock_log.exception.assert_any_call(expected_error_message)
        mock_messages.warning.assert_called_once_with(
            self.mock_request, expected_error_message)
        mock_log.exception.reset_mock()
        mock_messages.warning.reset_mock()

        # Test that second occurrence of HTTPConflict is caught.
        mock_package.requirements.return_value =\
            {'foo': mock.Mock(), 'bar': mock.Mock()}
        mock_murano_utils.Package.from_file.return_value = mock_package
        expected_error_message = _(
            "Package with specified name already exists")
        mock_api.muranoclient().packages.create.side_effect = [
            None, exc.HTTPConflict
        ]
        self.import_pkg_wizard.process_step(mock_form)
        mock_log.exception.assert_any_call(expected_error_message)
        mock_exc.handle.assert_any_call(
            self.mock_request, expected_error_message,
            redirect='test_redirect')

    @mock.patch.object(views, 'exceptions')
    @mock.patch.object(views, 'reverse')
    @mock.patch.object(views, 'LOG')
    @mock.patch.object(views, 'api')
    @mock.patch.object(views, 'muranoclient_utils')
    @mock.patch.object(views, 'glance')
    def test_process_step_except_http_internal_server_error(
            self, mock_glance, mock_murano_utils, mock_api, mock_log,
            mock_reverse, mock_exc):
        mock_murano_utils.Package.from_file.side_effect = None
        mock_reverse.return_value = 'test_redirect'
        mock_package = mock.Mock()
        mock_package.manifest = {'FullName': 'foo'}
        mock_package.requirements.return_value =\
            {'foo': mock.Mock(), 'bar': mock.Mock()}
        mock_murano_utils.Package.from_file.return_value = mock_package

        mock_form = mock.Mock()
        mock_form.cleaned_data = {
            'import_type': 'upload',
            'package': mock.Mock(file='test_package_file')
        }
        mock_form.data = {
            'dependencies': [],
            'images': []
        }

        expected_error_message = "Uploading package failed. {0}"\
                                 .format('test_500_error_message')
        exception = exc.HTTPInternalServerError(
            details='{"error": {"message": "test_500_error_message"}}')
        mock_api.muranoclient().packages.create.side_effect = [
            mock.Mock(id='test_package_id'), exception
        ]
        self.import_pkg_wizard.process_step(mock_form)
        mock_log.exception.assert_called_once_with(expected_error_message)
        mock_exc.handle.assert_called_once_with(
            self.mock_request, expected_error_message,
            redirect='test_redirect')
        mock_reverse.assert_called_once_with(
            'horizon:app-catalog:packages:index')

    @mock.patch.object(views, 'exceptions')
    @mock.patch.object(views, 'reverse')
    @mock.patch.object(views, 'muranodashboard_utils')
    @mock.patch.object(views, 'LOG')
    @mock.patch.object(views, 'api')
    @mock.patch.object(views, 'muranoclient_utils')
    @mock.patch.object(views, 'glance')
    def test_process_step_except_http_exception(
            self, mock_glance, mock_murano_utils, mock_api, mock_log,
            mock_dashboard_utils, mock_reverse, mock_exc):
        mock_murano_utils.Package.from_file.side_effect = None
        mock_reverse.return_value = 'test_redirect'
        mock_package = mock.Mock()
        mock_package.manifest = {'FullName': 'foo'}
        mock_package.requirements.return_value =\
            {'foo': mock.Mock(), 'bar': mock.Mock()}
        mock_murano_utils.Package.from_file.return_value = mock_package
        mock_dashboard_utils.parse_api_error.return_value = 'test_reason'

        mock_form = mock.Mock()
        mock_form.cleaned_data = {
            'import_type': 'upload',
            'package': mock.Mock(file='test_package_file')
        }
        mock_form.data = {
            'dependencies': [],
            'images': []
        }

        exception = exc.HTTPException(
            details='{"error": {"message": "test_error_message"}}')
        mock_api.muranoclient().packages.create.side_effect = [
            mock.Mock(id='test_package_id'), exception
        ]
        self.import_pkg_wizard.process_step(mock_form)
        mock_log.exception.assert_any_call('test_reason')
        mock_exc.handle.assert_called_once_with(
            self.mock_request, 'test_reason', redirect='test_redirect')
        mock_reverse.assert_called_once_with(
            'horizon:app-catalog:packages:index')

    def test_get_form_kwargs(self):
        kwargs = self.import_pkg_wizard.get_form_kwargs('add_category')
        self.assertEqual({'request': self.mock_request}, kwargs)

        mock_storage = mock.Mock()
        mock_storage.get_step_data().get.return_value = 'test_package'
        self.import_pkg_wizard.storage = mock_storage
        kwargs = self.import_pkg_wizard.get_form_kwargs('modify')
        self.assertEqual({'request': self.mock_request,
                          'package': 'test_package'}, kwargs)


class TestImportBundleWizard(helpers.RestAPITestCase):

    def setUp(self):
        super(TestImportBundleWizard, self).setUp()

        fake_response = {'status_code': 200}
        self.mock_request = mock.MagicMock(
            name='mock_request', return_value=fake_response, META=[])
        self.import_bundle_wizard = views.ImportBundleWizard()
        self.import_bundle_wizard.request = self.mock_request
        self.import_bundle_wizard.storage = mock.Mock(
            name='mock_storage', extra_data={'foo': 'bar'})
        self.import_bundle_wizard.steps = mock.Mock(name='mock_steps')
        self.import_bundle_wizard.steps.current = 'upload'
        self.import_bundle_wizard.prefix = 'test_prefix'
        self.import_bundle_wizard.initial_dict = {'upload': {'foo': 'bar'}}
        self.import_bundle_wizard.get_form_step_data =\
            lambda f: 'test_step_form_data'

        self.assertEqual('packages/import_bundle.html',
                         self.import_bundle_wizard.template_name)
        self.assertEqual('Import Bundle',
                         self.import_bundle_wizard.page_title)

        self.addCleanup(mock.patch.stopall)

    def test_get_context_data(self):
        mock_form = mock.Mock(initial={'package': mock.Mock(type='test_type')})
        context = self.import_bundle_wizard.get_context_data(form=mock_form)

        expected_instances = {
            'view': views.ImportBundleWizard,
            'wizard': dict
        }
        expected_context = {
            'foo': 'bar',
            'form': mock.ANY,
            'hide': True,
            'modal_backdrop': 'static',
            'murano_repo_url': 'http://apps.openstack.org',
            'view': mock.ANY,
            'wizard': mock.ANY
        }

        for key, val in expected_context.items():
            self.assertIn(key, context)
            self.assertEqual(val, context[key])

        for key, val in expected_instances.items():
            self.assertIsInstance(context[key], val)

    def test_get_form_initial(self):
        self.import_bundle_wizard.request.GET = {
            'url': 'test_url', 'name': 'test_name',
            'import_type': 'test_import_type'
        }
        initial_dict = self.import_bundle_wizard.get_form_initial('upload')
        expected_dict = {
            'foo': 'bar',
            'url': 'test_url',
            'name': 'test_name',
            'import_type': 'test_import_type'
        }

        for key, val in expected_dict.items():
            self.assertIn(key, initial_dict)
            self.assertEqual(val, initial_dict[key])

    @mock.patch.object(views, 'api')
    @mock.patch.object(views, 'muranoclient_utils')
    @mock.patch.object(views, 'glance')
    def test_process_step(self, mock_glance, mock_murano_utils, mock_api):
        mock_bundle = mock.Mock()
        mock_bundle.package_specs.return_value = [
            {'Name': 'foo_spec', 'Version': '1.0.0', 'Url': 'www.foo.com'},
            {'Name': 'bar_spec', 'Version': '2.0.0', 'Url': 'www.bar.com'}
        ]
        mock_package = mock.Mock()
        mock_foo_dependency = mock.Mock(name='foo_package')
        mock_bar_dependency = mock.Mock(name='bar_package')
        mock_package.requirements.return_value = {
            'foo_package': mock_foo_dependency,
            'bar_package': mock_bar_dependency
        }
        mock_murano_utils.to_url.return_value = 'test_url'
        mock_form = mock.Mock()

        for import_type in ('by_url', 'by_name'):
            mock_form.cleaned_data = {
                'import_type': import_type,
                'url': 'test_url',
                'name': 'test_form_name'
            }

            mock_murano_utils.Bundle.from_file.return_value = mock_bundle
            mock_murano_utils.Package.from_location.return_value = mock_package

            step_data = self.import_bundle_wizard.process_step(mock_form)
            self.assertEqual('test_step_form_data', step_data)

            mock_murano_utils.Bundle.from_file.assert_called_once_with(
                'test_url')
            mock_murano_utils.Package.from_location.assert_any_call(
                'foo_spec', version='1.0.0', url='www.foo.com',
                base_url=packages_consts.MURANO_REPO_URL, path=None
            )
            mock_murano_utils.Package.from_location.assert_any_call(
                'bar_spec', version='2.0.0', url='www.bar.com',
                base_url=packages_consts.MURANO_REPO_URL, path=None
            )
            mock_api.muranoclient().packages.create.assert_any_call(
                {}, {'foo_package': mock_foo_dependency.file()})
            mock_api.muranoclient().packages.create.assert_any_call(
                {}, {'bar_package': mock_bar_dependency.file()})

            mock_murano_utils.reset_mock()
            mock_api.reset_mock()

    @mock.patch.object(views, 'reverse')
    @mock.patch.object(views, 'messages')
    @mock.patch.object(views, 'LOG')
    @mock.patch.object(views, 'muranoclient_utils')
    def test_process_step_except_from_file_exception(
            self, mock_murano_utils, mock_log, mock_messages, mock_reverse):
        mock_reverse.return_value = 'test_redirect'
        mock_form = mock.Mock(
            cleaned_data={'import_type': 'by_url', 'url': 'foo_url'})
        e_404 = Exception('(404)')
        e_404.message = '(404)'
        e = Exception('foo')
        e.message = 'foo'
        errors = ((e_404, "Bundle creation failed.Reason: Can't find Bundle "
                          "name from repository."),
                  (e, "Bundle creation failed.Reason: foo"))

        for exception, expected_error_message in errors:
            mock_murano_utils.Bundle.from_file.side_effect = exception

            with self.assertRaisesRegex(horizon_exceptions.Http302, None):
                self.import_bundle_wizard.process_step(mock_form)

            mock_log.exception.assert_called_once_with(expected_error_message)
            mock_messages.error.assert_called_once_with(
                self.mock_request, expected_error_message)
            mock_reverse.assert_called_once_with(
                'horizon:app-catalog:packages:index')

            for mock_ in (mock_log, mock_messages, mock_reverse):
                mock_.reset_mock()

    @mock.patch.object(views, 'messages')
    @mock.patch.object(views, 'LOG')
    @mock.patch.object(views, 'muranoclient_utils')
    def test_process_step_except_from_location_exception(
            self, mock_murano_utils, mock_log, mock_messages):
        mock_form = mock.Mock(
            cleaned_data={'import_type': 'by_url', 'url': 'foo_url'})
        mock_bundle = mock.Mock()
        mock_bundle.package_specs.return_value = [
            {'Name': 'foo_spec', 'Version': '1.0.0', 'Url': 'www.foo.com'},
            {'Name': 'bar_spec', 'Version': '2.0.0', 'Url': 'www.bar.com'}
        ]

        mock_murano_utils.Bundle.from_file.return_value = mock_bundle
        mock_murano_utils.Package.from_location.side_effect = Exception('foo')

        self.import_bundle_wizard.process_step(mock_form)

        for spec in ('foo_spec', 'bar_spec'):
            expected_error_message = 'Error foo occurred while parsing '\
                                     'package {0}'.format(spec)
            mock_log.exception.assert_any_call(expected_error_message)
            mock_messages.error.assert_any_call(
                self.mock_request, expected_error_message)

    @mock.patch.object(views, 'messages')
    @mock.patch.object(views, 'LOG')
    @mock.patch.object(views, 'api')
    @mock.patch.object(views, 'muranoclient_utils')
    @mock.patch.object(views, 'glance')
    def test_process_step_except_http_conflict(
            self, mock_glance, mock_murano_utils, mock_api, mock_log,
            mock_messages):
        mock_form = mock.Mock(
            cleaned_data={'import_type': 'by_url', 'url': 'foo_url'})
        mock_bundle = mock.Mock()
        mock_bundle.package_specs.return_value = [
            {'Name': 'foo_spec', 'Version': '1.0.0', 'Url': 'www.foo.com'},
            {'Name': 'bar_spec', 'Version': '2.0.0', 'Url': 'www.bar.com'}
        ]
        mock_foo_dependency = mock.Mock(name='foo_package')
        mock_bar_dependency = mock.Mock(name='bar_package')
        mock_package = mock.Mock()
        mock_package.requirements.return_value = {
            'foo_package': mock_foo_dependency,
            'bar_package': mock_bar_dependency
        }

        mock_murano_utils.Bundle.from_file.return_value = mock_bundle
        mock_murano_utils.Package.from_location.return_value = mock_package
        mock_api.muranoclient().packages.create.side_effect = exc.HTTPConflict

        self.import_bundle_wizard.process_step(mock_form)

        for dep in ('foo_package', 'bar_package'):
            expected_error_message = 'Package {0} already registered.'\
                                     .format(dep)
            mock_log.exception.assert_any_call(expected_error_message)
            mock_messages.warning.assert_any_call(
                self.mock_request, expected_error_message)

    @mock.patch.object(views, 'messages')
    @mock.patch.object(views, 'LOG')
    @mock.patch.object(views, 'api')
    @mock.patch.object(views, 'muranodashboard_utils')
    @mock.patch.object(views, 'muranoclient_utils')
    @mock.patch.object(views, 'glance')
    def test_process_step_except_http_exception(
            self, mock_glance, mock_murano_utils, mock_dashboard_utils,
            mock_api, mock_log, mock_messages):
        mock_form = mock.Mock(
            cleaned_data={'import_type': 'by_url', 'url': 'foo_url'})
        mock_bundle = mock.Mock()
        mock_bundle.package_specs.return_value = [
            {'Name': 'foo_spec', 'Version': '1.0.0', 'Url': 'www.foo.com'},
            {'Name': 'bar_spec', 'Version': '2.0.0', 'Url': 'www.bar.com'}
        ]
        mock_foo_dependency = mock.Mock(name='foo_package')
        mock_bar_dependency = mock.Mock(name='bar_package')
        mock_package = mock.Mock()
        mock_package.requirements.return_value = {
            'foo_package': mock_foo_dependency,
            'bar_package': mock_bar_dependency
        }

        mock_murano_utils.Bundle.from_file.return_value = mock_bundle
        mock_murano_utils.Package.from_location.return_value = mock_package
        mock_api.muranoclient().packages.create.side_effect =\
            exc.HTTPException('foo')
        mock_dashboard_utils.parse_api_error.return_value = 'foo'

        self.import_bundle_wizard.process_step(mock_form)

        for dep in ('foo_package', 'bar_package'):
            expected_error_message = 'Package {0} upload failed. foo'\
                                     .format(dep)
            mock_log.exception.assert_any_call(expected_error_message)
            mock_messages.warning.assert_any_call(
                self.mock_request, expected_error_message)

    @mock.patch.object(views, 'messages')
    @mock.patch.object(views, 'LOG')
    @mock.patch.object(views, 'api')
    @mock.patch.object(views, 'muranoclient_utils')
    @mock.patch.object(views, 'glance')
    def test_process_step_except_package_create_exception(
            self, mock_glance, mock_murano_utils, mock_api, mock_log,
            mock_messages):
        mock_form = mock.Mock(
            cleaned_data={'import_type': 'by_url', 'url': 'foo_url'})
        mock_bundle = mock.Mock()
        mock_bundle.package_specs.return_value = [
            {'Name': 'foo_spec', 'Version': '1.0.0', 'Url': 'www.foo.com'},
            {'Name': 'bar_spec', 'Version': '2.0.0', 'Url': 'www.bar.com'}
        ]
        mock_foo_dependency = mock.Mock(name='foo_package')
        mock_bar_dependency = mock.Mock(name='bar_package')
        mock_package = mock.Mock()
        mock_package.requirements.return_value = {
            'foo_package': mock_foo_dependency,
            'bar_package': mock_bar_dependency
        }

        mock_murano_utils.Bundle.from_file.return_value = mock_bundle
        mock_murano_utils.Package.from_location.return_value = mock_package
        mock_api.muranoclient().packages.create.side_effect =\
            Exception('foo')

        self.import_bundle_wizard.process_step(mock_form)

        for dep in ('foo_package', 'bar_package'):
            expected_error_message = 'Importing package {0} failed. '\
                                     'Reason: foo'.format(dep)
            mock_log.exception.assert_any_call(expected_error_message)
            mock_messages.warning.assert_any_call(
                self.mock_request, expected_error_message)

    @mock.patch.object(views, 'messages')
    @mock.patch.object(views, 'LOG')
    @mock.patch.object(views, 'reverse')
    def test_done(self, mock_reverse, mock_log, mock_messages):
        mock_reverse.redirect = 'test_redirect'

        result = self.import_bundle_wizard.done([])
        self.assertIsInstance(result, http.response.HttpResponseRedirect)

        expected_message = 'Bundle successfully imported.'
        mock_log.info.assert_any_call(expected_message)
        mock_messages.success.assert_called_once_with(
            self.mock_request, expected_message)
        mock_reverse.assert_called_once_with(
            'horizon:app-catalog:packages:index')


class TestPackageDefinitionsView(helpers.APITestCase):

    def setUp(self):
        super(TestPackageDefinitionsView, self).setUp()

        self.pkg_definitions_view = views.PackageDefinitionsView()
        mock_token = mock.MagicMock()
        mock_token.__getitem__.return_value = 'foo_token_id'
        self.mock_request = mock.MagicMock(
            name='mock_request', GET={'sort_dir': 'asc'},
            session={'token': mock_token})

        self.pkg_definitions_view.request = self.mock_request
        self.original_get_filters = self.pkg_definitions_view.get_filters
        self.pkg_definitions_view.get_filters = lambda opts: opts

        self.assertEqual(self.pkg_definitions_view.table_class,
                         tables.PackageDefinitionsTable)
        self.assertEqual(self.pkg_definitions_view.template_name,
                         'packages/index.html')
        self.assertEqual(self.pkg_definitions_view.page_title, 'Packages')
        self.assertFalse(self.pkg_definitions_view.has_more_data(None))
        self.assertFalse(self.pkg_definitions_view.has_prev_data(None))

        mock_horizon_utils = mock.patch.object(views, 'utils').start()
        mock_horizon_utils.get_page_size.return_value = 123
        self.addCleanup(mock.patch.stopall)

    @mock.patch.object(views, 'pkg_api')
    def test_get_data_with_same_tenants(self, mock_pkg_api):
        mock_package = mock.Mock(
            id='foo_package', owner_id='test_tenant', tenant_name=None)
        mock_pkg_api.package_list.return_value =\
            ([mock_package], True)
        mock_tenant = mock.MagicMock()
        mock_tenant.__getitem__.side_effect = [
            'test_tenant', 'foo_tenant_name'
        ]

        self.pkg_definitions_view.request.user.is_superuser = False
        self.pkg_definitions_view.request.session['token'] =\
            mock.Mock(tenant=mock_tenant)

        packages = self.pkg_definitions_view.get_data()

        self.assertEqual([mock_package], packages)
        self.assertEqual('foo_tenant_name', mock_package.tenant_name)
        self.assertTrue(self.pkg_definitions_view.has_prev_data)
        mock_pkg_api.package_list.assert_any_call(
            self.mock_request, marker=None,
            filters={'include_disabled': True, 'sort_dir': 'desc'},
            paginate=True, page_size=123)
        mock_pkg_api.package_list.assert_any_call(
            self.mock_request, marker='foo_package',
            filters={'include_disabled': True, 'sort_dir': 'desc'},
            paginate=True, page_size=0)

    @mock.patch.object(views, 'pkg_api')
    def test_get_data_with_different_tenants(self, mock_pkg_api):
        mock_package = mock.Mock(id='foo_package', tenant_name='test_tenant')
        mock_pkg_api.package_list.return_value =\
            ([mock_package], True)

        self.pkg_definitions_view.request.user.is_superuser = False
        self.pkg_definitions_view.request.session['token'] =\
            mock.MagicMock(__getitem__='alt_test_tenant')

        packages = self.pkg_definitions_view.get_data()

        self.assertEqual([mock_package], packages)
        self.assertEqual('UNKNOWN', mock_package.tenant_name)
        self.assertTrue(self.pkg_definitions_view.has_prev_data)
        mock_pkg_api.package_list.assert_any_call(
            self.mock_request, marker=None,
            filters={'include_disabled': True, 'sort_dir': 'desc'},
            paginate=True, page_size=123)
        mock_pkg_api.package_list.assert_any_call(
            self.mock_request, marker='foo_package',
            filters={'include_disabled': True, 'sort_dir': 'desc'},
            paginate=True, page_size=0)

    @mock.patch.object(views, 'keystone')
    @mock.patch.object(views, 'pkg_api')
    def test_get_data_as_superuser(self, mock_pkg_api, mock_keystone):
        mock_keystone.tenant_list.side_effect = None
        super_user_tenant = mock.Mock(id='super_tenant_id')
        super_user_tenant.configure_mock(name='super_tenant_name')
        mock_keystone.tenant_list.return_value = ([super_user_tenant], False)
        mock_package = mock.Mock(
            id='foo_package', owner_id='super_tenant_id', tenant_name=None)
        mock_pkg_api.package_list.return_value =\
            ([mock_package], True)
        mock_tenant = mock.MagicMock()
        mock_tenant.__getitem__.side_effect = [
            'test_tenant', 'foo_tenant_name'
        ]

        self.pkg_definitions_view.request.user.is_superuser = True
        self.pkg_definitions_view.request.session['token'] =\
            mock.Mock(tenant=mock_tenant)

        packages = self.pkg_definitions_view.get_data()

        self.assertEqual([mock_package], packages)
        self.assertEqual('super_tenant_name', mock_package.tenant_name)
        self.assertTrue(self.pkg_definitions_view.has_more_data)

    @mock.patch.object(views, 'exceptions')
    @mock.patch.object(views, 'keystone')
    @mock.patch.object(views, 'pkg_api')
    def test_get_data_except_keystone_exception(self, mock_pkg_api,
                                                mock_keystone, mock_exc):
        mock_keystone.tenant_list.side_effect = Exception
        super_user_tenant = mock.Mock(id='super_tenant_id')
        super_user_tenant.configure_mock(name='super_tenant_name')
        mock_keystone.tenant_list.return_value = ([super_user_tenant], False)
        mock_package = mock.Mock(
            id='foo_package', owner_id='super_tenant_id', tenant_name=None)
        mock_pkg_api.package_list.return_value =\
            ([mock_package], True)
        mock_tenant = mock.MagicMock()
        mock_tenant.__getitem__.side_effect = [
            'test_tenant', 'foo_tenant_name'
        ]

        self.pkg_definitions_view.request.user.is_superuser = True
        self.pkg_definitions_view.request.session['token'] =\
            mock.Mock(tenant=mock_tenant)

        packages = self.pkg_definitions_view.get_data()

        self.assertEqual([mock_package], packages)
        self.assertIsNone(mock_package.tenant_name)
        self.assertTrue(self.pkg_definitions_view.has_more_data)
        mock_exc.handle.assert_called_once_with(
            self.mock_request, "Unable to retrieve project list.")

    @mock.patch.object(views, 'pkg_api')
    def test_get_data_desc_order(self, mock_pkg_api):
        mock_package = mock.Mock(
            id='foo_package', owner_id='test_tenant', tenant_name=None)
        mock_pkg_api.package_list.return_value =\
            ([mock_package], True)
        mock_tenant = mock.MagicMock()
        mock_tenant.__getitem__.side_effect = [
            'test_tenant', 'foo_tenant_name'
        ]

        self.pkg_definitions_view.request.GET = {'sort_dir': 'desc'}
        self.pkg_definitions_view.request.user.is_superuser = False
        self.pkg_definitions_view.request.session['token'] =\
            mock.Mock(tenant=mock_tenant)

        packages = self.pkg_definitions_view.get_data()

        self.assertEqual([mock_package], packages)
        self.assertEqual('foo_tenant_name', mock_package.tenant_name)
        self.assertTrue(self.pkg_definitions_view.has_more_data)

        mock_pkg_api.package_list.assert_any_call(
            self.mock_request, marker=None,
            filters={'include_disabled': True, 'sort_dir': 'asc'},
            paginate=True, page_size=123)
        mock_pkg_api.package_list.assert_any_call(
            self.mock_request, marker='foo_package',
            filters={'include_disabled': True, 'sort_dir': 'asc'},
            paginate=True, page_size=0)

    def test_get_context_data(self):
        mock_form = mock.Mock(initial={'package': mock.Mock(type='test_type')})
        self.pkg_definitions_view.table = 'foo_table'
        context = self.pkg_definitions_view.get_context_data(form=mock_form)

        expected_context = {
            'table': 'foo_table',
            'tenant_id': mock.ANY,
            'packages_table': 'foo_table',
            'form': mock.ANY,
            'view': mock.ANY
        }

        for key, val in expected_context.items():
            self.assertEqual(val, context[key])

        self.assertIsInstance(context['view'], views.PackageDefinitionsView)

    def test_get_filters(self):
        mock_filter_action = mock.Mock()
        mock_filter_action.is_api_filter.return_value = True
        self.pkg_definitions_view.table = mock.Mock()
        self.pkg_definitions_view.table._meta_._filter_action =\
            mock_filter_action
        self.pkg_definitions_view.table.get_filter_field.return_value =\
            'test_filter_field'
        self.pkg_definitions_view.table.get_filter_string.return_value =\
            'test_filter_string'

        self.pkg_definitions_view.get_filters = self.original_get_filters
        test_filters = {'foo': 'bar'}
        filters = self.pkg_definitions_view.get_filters(test_filters)

        expected_filters = {
            'test_filter_field': 'test_filter_string',
            'foo': 'bar'
        }

        for key, val in expected_filters.items():
            self.assertEqual(val, filters[key])



# Copyright (c) 2016 AT&T Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import collections
import unittest
from unittest import mock

from django.conf import settings
from django.forms import formsets
from django import http
from django.utils.translation import gettext_lazy as _

from horizon.forms import views as horizon_views

from muranoclient.common import exceptions as exc

from muranodashboard.catalog import tabs as catalog_tabs
from muranodashboard.catalog import views

try:
    from urlparse import urlparse
except ImportError:
    from urllib.parse import urlparse


class TestCatalogViews(unittest.TestCase):
    def setUp(self):
        super(TestCatalogViews, self).setUp()
        self.mock_request = mock.MagicMock(session={})
        self.env = mock.MagicMock(id='12', name='test_env', status='READY')

        self.addCleanup(mock.patch.stopall)

    def test_is_valid_environment(self):
        mock_env1 = mock.MagicMock(id='13')
        valid_envs = [mock_env1, self.env]
        self.assertTrue(views.is_valid_environment(self.env, valid_envs))

    @mock.patch.object(views, 'env_api')
    def test_get_environments_context(self, mock_env_api):
        mock_env_api.environments_list.return_value = [self.env]
        self.assertIsNotNone(views.get_environments_context(self.mock_request))
        mock_env_api.environments_list.assert_called_with(self.mock_request)

    @mock.patch.object(views, 'api')
    def test_get_categories_list(self, mock_api):
        self.assertEqual([], views.get_categories_list(self.mock_request))
        mock_api.handled_exceptions.assert_called_once_with(self.mock_request)
        mock_api.muranoclient.assert_called_once_with(self.mock_request)

    @mock.patch.object(views, 'env_api')
    def test_create_quick_environment(self, mock_env_api):
        views.create_quick_environment(self.mock_request)
        self.assertTrue(mock_env_api.environment_create.called)

    @mock.patch.object(views, 'pkg_api')
    def test_get_image(self, mock_pkg_api):
        app_id = 13
        result = views.get_image(self.mock_request, app_id)
        self.assertIsInstance(result, http.HttpResponse)
        (mock_pkg_api.get_app_logo.
         assert_called_once_with(self.mock_request, app_id))
        mock_pkg_api.reset_mock()

        mock_pkg_api.get_app_logo.return_value = None
        result = views.get_image(self.mock_request, app_id)
        self.assertIsInstance(result, http.HttpResponseRedirect)
        (mock_pkg_api.get_app_logo.
         assert_called_once_with(self.mock_request, app_id))

    @mock.patch.object(views, 'pkg_api')
    def test_get_supplier_image(self, mock_pkg_api):
        app_id = 13
        result = views.get_supplier_image(self.mock_request, app_id)
        self.assertIsInstance(result, http.HttpResponse)
        (mock_pkg_api.get_app_supplier_logo.
         assert_called_once_with(self.mock_request, app_id))
        mock_pkg_api.reset_mock()

        mock_pkg_api.get_app_supplier_logo.return_value = None
        result = views.get_supplier_image(self.mock_request, app_id)
        self.assertIsInstance(result, http.HttpResponseRedirect)
        (mock_pkg_api.get_app_supplier_logo.
         assert_called_once_with(self.mock_request, app_id))

    @mock.patch.object(views, 'pkg_api')
    @mock.patch('muranodashboard.dynamic_ui.services.version')
    @mock.patch('muranodashboard.dynamic_ui.services.pkg_api')
    def test_quick_deploy_error(self, services_pkg_api,
                                mock_version, views_pkg_api):
        mock_version.check_version.return_value = '0.2'
        app_id = 'app_id'
        self.assertRaises(ValueError, views.quick_deploy,
                          self.mock_request, app_id=app_id)
        (services_pkg_api.get_app_ui.
         assert_called_once_with(self.mock_request, app_id))
        views_pkg_api.get_app_fqn.assert_called_with(self.mock_request, app_id)

    @mock.patch.object(views, 'shortcuts')
    @mock.patch.object(views, 'get_available_environments')
    @mock.patch.object(views, 'http_utils')
    def test_switch(self, mock_http_utls, mock_get_available_environments,
                    mock_shortcuts):
        mock_http_utls.is_safe_url.return_value = True
        self.mock_request.GET = {'redirect': 'redirect_to_foo'}
        mock_env = mock.Mock(id='foo_env_id')
        mock_get_available_environments.return_value = [mock_env]
        mock_shortcuts.redirect.return_value = 'foo_redirect'

        result = views.switch(self.mock_request, 'foo_env_id',
                              redirect_field_name='redirect')
        self.assertEqual('foo_redirect', result)
        self.assertEqual(mock_env, self.mock_request.session['environment'])
        mock_shortcuts.redirect.assert_called_once_with('redirect_to_foo')
        mock_shortcuts.redirect.reset_mock()

        mock_http_utls.url_has_allowed_host_and_scheme.return_value = False
        result = views.switch(self.mock_request, 'foo_env_id',
                              redirect_field_name='redirect')
        self.assertEqual('foo_redirect', result)
        self.assertEqual(mock_env, self.mock_request.session['environment'])
        mock_shortcuts.redirect.assert_called_once_with(
            settings.LOGIN_REDIRECT_URL)

    @mock.patch.object(views, 'env_api')
    def test_get_next_quick_environment_name(self, mock_env_api):
        # Test whether non-matching name is not returned.
        match_env = mock.Mock()
        match_env.configure_mock(name='quick-env-123')
        non_match_env = mock.Mock()
        non_match_env.configure_mock(name='quick-env-foo')
        mock_env_api.environments_list.return_value = [
            match_env, non_match_env
        ]
        result = views.get_next_quick_environment_name(self.mock_request)
        self.assertEqual('quick-env-124', result)

        # Test whether matching name with biggest number is returned.
        non_match_env.configure_mock(name='quick-env-124')
        mock_env_api.environments_list.return_value = [
            match_env, non_match_env
        ]
        result = views.get_next_quick_environment_name(self.mock_request)
        self.assertEqual('quick-env-125', result)

    @mock.patch.object(views, 'api')
    def test_cleaned_latest_apps(self, mock_api):
        foo_app = mock.Mock(id='foo_id')
        bar_app = mock.Mock(id='bar_id')
        mock_api.muranoclient().packages.filter.return_value = [
            foo_app, bar_app
        ]
        self.mock_request.session['latest_apps'] = ['foo', 'bar']

        expected_params = {
            'type': 'Application',
            'catalog': True,
            'id': 'in:foo,bar'
        }

        result = views.cleaned_latest_apps(self.mock_request)
        self.assertEqual([foo_app, bar_app], result)
        self.assertEqual(collections.deque(['foo_id', 'bar_id']),
                         self.mock_request.session['latest_apps'])
        mock_api.muranoclient().packages.filter.assert_called_once_with(
            **expected_params)


class TestLazyWizard(unittest.TestCase):

    @mock.patch.object(views.LazyWizard, 'http_method_names',
                       new_callable=mock.PropertyMock)
    def test_as_view_except_type_error(self, mock_http_method_names):
        mock_http_method_names.return_value = ['patch']

        # Test that first occurrence of type error is thrown.
        kwargs = {'patch': ''}
        expected_error_msg = "You tried to pass in the {0} method name as a "\
                             "keyword argument to LazyWizard(). "\
                             "Don't do that.".format("patch")

        with self.assertRaises(TypeError) as cm:
            views.LazyWizard.as_view(None, **kwargs)
        e = cm.exception
        self.assertEqual(expected_error_msg, str(e))

        # Test that second occurrence of type error is thrown.
        kwargs = {'foobar': ''}
        expected_error_msg = "LazyWizard() received an invalid keyword "\
                             "'foobar'"

        with self.assertRaises(TypeError) as cm:
            views.LazyWizard.as_view(None, **kwargs)
        e = cm.exception
        self.assertEqual(expected_error_msg, str(e))

    @mock.patch.object(views.LazyWizard, 'dispatch')
    def test_as_view(self, mock_dispatch):
        form = mock.Mock()
        form.__name__ = 'test_form'
        form.base_fields = {'foo': None, 'bar': None}
        formset = formsets.formset_factory(form)

        mock_request = mock.Mock()
        mock_request.session = {}
        mock_initforms = mock.Mock(return_value=[formset])
        kwargs = {'app_id': 'foo'}
        view = views.LazyWizard.as_view(mock_initforms)

        self.assertTrue(hasattr(view, '__call__'))
        view(mock_request, **kwargs)
        mock_initforms.assert_called_once_with(mock_request, kwargs)
        mock_dispatch.assert_called_once_with(mock_request, **kwargs)


class TestWizard(unittest.TestCase):

    def setUp(self):
        super(TestWizard, self).setUp()
        self.wizard = views.Wizard()
        self.wizard.storage = mock.MagicMock()
        self.wizard.storage.extra_data.__getitem__().name = 'foo_app'
        self.wizard.kwargs = {
            'do_redirect': 'redirect_to_foo',
            'environment_id': 'foo_env_id',
            'app_id': 'foo_app_id'
        }
        self.wizard.request = mock.Mock()
        self.wizard.request.META = {}
        self.wizard.request.session = {'quick_env_id': 'quick_foo_env_id'}
        self.assertEqual('services/wizard_create.html',
                         self.wizard.template_name)

        self.assertEqual(False, self.wizard.do_redirect)
        self.assertEqual('Add Application', self.wizard.page_title)

    def test_get_prefix(self):
        kwargs = {'foo': 'bar'}
        prefix = self.wizard.get_prefix(None, **kwargs)
        self.assertIsInstance(prefix, str)

    def test_get_form_prefix(self):
        self.wizard.steps = mock.Mock()
        self.wizard.steps.step0 = 'foo_step'
        self.wizard.steps.all.index.return_value = 'bar_step'
        self.assertEqual('foo_step', self.wizard.get_form_prefix())
        self.assertEqual('bar_step', self.wizard.get_form_prefix(step='bar'))

    @mock.patch.object(views, 'messages')
    @mock.patch.object(views, 'LOG')
    @mock.patch.object(views, 'env_api')
    @mock.patch.object(views, 'reverse')
    def test_done(self, mock_reverse, mock_env_api, mock_log, mock_messages):
        mock_service = mock.Mock()
        mock_service.service.extract_attributes.return_value = {'foo': 'bar'}
        mock_created_service = mock.Mock()
        setattr(mock_created_service, '?', {'id': 'foo_service_id'})
        mock_env_api.service_create.return_value = mock_created_service

        self.wizard.done([mock_service])

        mock_reverse.assert_any_call(
            "horizon:app-catalog:environments:index")
        mock_reverse.assert_any_call(
            "horizon:app-catalog:environments:services",
            args=('quick_foo_env_id',))
        mock_env_api.service_create.assert_called_once_with(
            self.wizard.request, 'quick_foo_env_id', mock.ANY)

        expected_message = "The 'foo_app' application successfully added to "\
            "environment."
        mock_log.info.assert_called_once_with(expected_message)
        mock_messages.success.assert_called_once_with(
            self.wizard.request, expected_message)

    @mock.patch.object(views, 'exceptions')
    @mock.patch.object(views, 'env_api')
    @mock.patch.object(views, 'reverse')
    def test_done_except_http_forbidden(self, mock_reverse, mock_env_api,
                                        mock_exceptions):
        mock_reverse.return_value = 'foo_url'
        mock_service = mock.Mock()
        mock_service.service.extract_attributes.return_value = {'foo': 'bar'}
        mock_env_api.service_create.side_effect = exc.HTTPForbidden

        self.wizard.done([mock_service])

        expected_error_msg = _("Sorry, you can't add application right now. "
                               "The environment is deploying.")
        mock_exceptions.handle.assert_called_once_with(
            self.wizard.request, expected_error_msg, redirect='foo_url')

    @mock.patch.object(views, 'exceptions')
    @mock.patch.object(views, 'LOG')
    @mock.patch.object(views, 'env_api')
    @mock.patch.object(views, 'reverse')
    def test_done_except_exception(self, mock_reverse, mock_env_api, mock_log,
                                   mock_exceptions):
        mock_reverse.return_value = 'foo_url'
        mock_service = mock.Mock()
        mock_service.service.extract_attributes.return_value = {'foo': 'bar'}
        mock_env_api.service_create.side_effect = Exception

        self.wizard.done([mock_service])

        expected_error_msg = _('Adding application to an environment failed.')
        mock_log.exception(expected_error_msg)
        mock_env_api.environment_delete.assert_called_once_with(
            self.wizard.request, 'quick_foo_env_id')
        mock_exceptions.handle.assert_called_once_with(
            self.wizard.request, expected_error_msg, redirect='foo_url')

    def test_create_hacked_response(self):
        self.wizard.request.META = {
            horizon_views.ADD_TO_FIELD_HEADER: 'foo_field_id'
        }
        response = self.wizard.create_hacked_response(
            'foo_obj_id', 'foo_obj_name')
        self.assertIsInstance(response, http.HttpResponse)
        self.assertEqual(b'["foo_obj_id", "foo_obj_name"]', response.content)
        self.assertTrue(response.has_header('X-Horizon-Add-To-Field'))
        self.assertEqual('foo_field_id',
                         response['X-Horizon-Add-To-Field'])

    def test_create_hacked_response_empty_response(self):
        response = self.wizard.create_hacked_response(None, None)
        self.assertIsInstance(response, http.HttpResponse)
        self.assertEqual(b'', response.content)

    @mock.patch.object(views, 'utils')
    def test_get_form_initial(self, mock_utils):
        mock_utils.ensure_python_obj.return_value = 'foo_env_id'
        self.wizard.initial_dict = {'foo_step': 'foo'}

        # Test whether correct dict entry is returned.
        result = self.wizard.get_form_initial('foo_step')
        self.assertEqual('foo', result)
        mock_utils.ensure_python_obj.assert_called_once_with('foo_env_id')

        # Test whether init_dict returned because key not found.
        result = self.wizard.get_form_initial('bar_step')
        expected = {
            'request': self.wizard.request,
            'app_id': 'foo_app_id',
            'environment_id': 'foo_env_id'
        }
        for key, val in expected.items():
            self.assertEqual(val, result[key])

        # Test whether alternate env_id is used.
        mock_utils.ensure_python_obj.return_value = None
        result = self.wizard.get_form_initial('bar_step')
        expected = {
            'request': self.wizard.request,
            'app_id': 'foo_app_id',
            'environment_id': 'quick_foo_env_id'
        }
        for key, val in expected.items():
            self.assertEqual(val, result[key])

    @mock.patch.object(
        views, 'nova',
        mock.MagicMock(side_effect=views.nova_exceptions.ClientException))
    def test_get_flavors(self):
        result = self.wizard.get_flavors()

        self.assertEqual('[]', result)
        views.nova.flavor_list.assert_called_once_with(self.wizard.request)

    @mock.patch.object(views, 'nova')
    @mock.patch.object(views, 'quotas')
    @mock.patch.object(views, 'services')
    @mock.patch.object(views, 'api')
    def test_get_context_data(self, mock_api, mock_services, mock_quotas,
                              mock_nova):
        mock_api.muranoclient().environments.get().name = 'foo_env_name'
        mock_services.get_app_field_descriptions.return_value = [
            'foo_field_descr', 'foo_extended_descr'
        ]
        mock_nova.flavor_list.return_value = [
            type('FakeFlavor%s' % k, (object, ),
                 {'id': 'fake_id_%s' % k, 'name': 'fake_name_%s' % k,
                  '_info': {'foo': 'bar'}})
            for k in (1, 2)
        ]

        form = mock.Mock()
        app = mock.Mock(fully_qualified_name='foo_app_fqn')
        app.configure_mock(name='foo_app')

        self.wizard.request.GET = {}
        self.wizard.request.POST = {}
        self.wizard.storage.extra_data.get.return_value = app
        self.wizard.steps = mock.Mock(index='foo_step_index', step0=-1)
        self.wizard.prefix = 'foo_prefix'
        self.wizard.kwargs['do_redirect'] = 'foo_do_redirect'
        self.wizard.kwargs['drop_wm_form'] = 'foo_drop_wm_form'
        context = self.wizard.get_context_data(form)

        expected_context = {
            'type': 'foo_app_fqn',
            'service_name': 'foo_app',
            'app_id': 'foo_app_id',
            'environment_id': 'foo_env_id',
            'environment_name': 'foo_env_name',
            'do_redirect': 'foo_do_redirect',
            'drop_wm_form': 'foo_drop_wm_form',
            'prefix': 'foo_prefix',
            'wizard_id': mock.ANY,
            'field_descriptions': 'foo_field_descr',
            'extended_descriptions': 'foo_extended_descr'
        }
        for key, val in expected_context.items():
            self.assertIn(key, context)
            self.assertEqual(val, context[key])

        mock_api.muranoclient().environments.get.assert_called_with(
            'foo_env_id')
        mock_services.get_app_field_descriptions.assert_called_once_with(
            self.wizard.request, 'foo_app_id', 'foo_step_index')
        mock_nova.flavor_list.assert_called_once_with(self.wizard.request)

    @mock.patch.object(views, 'nova')
    @mock.patch.object(views, 'quotas')
    @mock.patch.object(views, 'env_api')
    @mock.patch.object(views, 'utils')
    @mock.patch.object(views, 'services')
    @mock.patch.object(views, 'api')
    def test_get_context_data_alternate_control_flow(
            self, mock_api, mock_services, mock_utils, mock_env_api,
            mock_quatas, mock_nova):
        form = mock.Mock()
        app = mock.Mock(fully_qualified_name='foo_app_fqn')
        app.configure_mock(name='foo_app')

        mock_api.muranoclient().environments.get().name = 'foo_env_name'
        mock_api.muranoclient().packages.get.return_value = app
        mock_services.get_app_field_descriptions.return_value = [
            'foo_field_descr', 'foo_extended_descr'
        ]
        mock_utils.ensure_python_obj.return_value = None
        mock_env_api.environments_list.return_value = []
        mock_nova.flavor_list.return_value = [
            type('FakeFlavor%s' % k, (object, ),
                 {'id': 'fake_id_%s' % k, 'name': 'fake_name_%s' % k,
                  '_info': {'foo': 'bar'}})
            for k in (1, 2)
        ]

        self.wizard.request.GET = {}
        self.wizard.request.POST = {'wizard_id': 'foo_wizard_id'}
        self.wizard.storage.extra_data = {}
        self.wizard.steps = mock.Mock(index='foo_step_index', step0=0)
        self.wizard.steps.all = []
        self.wizard.prefix = 'foo_prefix'
        context = self.wizard.get_context_data(form)

        expected_context = {
            'type': 'foo_app_fqn',
            'service_name': 'foo_app',
            'app_id': 'foo_app_id',
            'environment_id': None,
            'environment_name': 'quick-env-1',
            'do_redirect': mock.ANY,
            'drop_wm_form': mock.ANY,
            'prefix': 'foo_prefix',
            'wizard_id': 'foo_wizard_id',
            'field_descriptions': 'foo_field_descr',
            'extended_descriptions': 'foo_extended_descr'
        }
        for key, val in expected_context.items():
            self.assertIn(key, context)
            self.assertEqual(val, context[key])
        self.assertEqual(app, self.wizard.storage.extra_data['app'])

        mock_api.muranoclient().packages.get.assert_called_once_with(
            'foo_app_id')
        mock_api.muranoclient().environments.get.assert_called_once_with()
        mock_services.get_app_field_descriptions.assert_called_once_with(
            self.wizard.request, 'foo_app_id', 'foo_step_index')
        mock_nova.flavor_list.assert_called_once_with(self.wizard.request)


class TestIndexView(unittest.TestCase):

    def setUp(self):
        super(TestIndexView, self).setUp()
        self.index_view = views.IndexView()

        self.assertEqual(6, self.index_view.paginate_by)
        self.assertEqual(_("Browse"), self.index_view.page_title)
        self.assertIsNone(self.index_view._more)

        self.index_view.request = mock.Mock()
        self.index_view.request.GET = {
            'category': 'foo_category',
            'search': 'foo_search',
            'order_by': 'foo_col',
            'sort_dir': 'desc',
            'marker': 'foo_marker'
        }

    def test_get_object_id(self):
        mock_datum = mock.Mock(id='foo_datum_id')
        self.assertEqual('foo_datum_id',
                         views.IndexView.get_object_id(mock_datum))

    def test_get_marker(self):
        mock_datum = mock.Mock(id='foo_datum_id')
        self.index_view.object_list = [mock_datum]
        self.assertEqual('foo_datum_id', self.index_view.get_marker())

        self.index_view.object_list = []
        self.assertEqual('', self.index_view.get_marker())

    def test_get_query_params(self):
        expected = {
            'search': 'foo_search',
            'order_by': 'foo_col',
            'sort_dir': 'desc'
        }

        query_params = self.index_view.get_query_params(internal_query=False)
        for key, val in expected.items():
            self.assertEqual(val, query_params[key])

        expected['type'] = 'Application'
        query_params = self.index_view.get_query_params(internal_query=True)
        for key, val in expected.items():
            self.assertEqual(val, query_params[key])

    @mock.patch.object(views, 'pkg_api')
    def test_get_queryset(self, mock_pkg_api):
        mock_pkg_api.package_list.return_value = (
            ['bar_pkg', 'foo_pkg'], False)

        self.index_view.paginate_by = 123
        packages = self.index_view.get_queryset()

        self.assertEqual(['foo_pkg', 'bar_pkg'], packages)
        self.assertFalse(self.index_view._more)
        mock_pkg_api.package_list.assert_called_once_with(
            self.index_view.request, filters=mock.ANY, paginate=True,
            marker='foo_marker', page_size=123, sort_dir='desc',
            limit=123)

    def test_get_template_names(self):
        self.assertEqual(['catalog/index.html'],
                         self.index_view.get_template_names())

    def test_has_next_page(self):
        self.index_view.request.GET = {'sort_dir': 'asc'}
        self.index_view._more = False
        self.assertFalse(self.index_view.has_next_page())

    @mock.patch.object(views, 'pkg_api')
    def test_has_next_page_with_api_query(self, mock_pkg_api):
        mock_pkg_api.package_list.return_value = (['foo'], False)

        self.index_view.get_marker = lambda: 'foo_marker'
        result = self.index_view.has_next_page()

        self.assertTrue(result)
        mock_pkg_api.package_list.assert_called_once_with(
            self.index_view.request, filters=mock.ANY, paginate=True,
            marker='foo_marker', page_size=1)

    def test_has_prev_page(self):
        self.index_view._more = False
        self.index_view.request.GET = {'sort_dir': 'desc'}
        self.assertFalse(self.index_view.has_prev_page())

        self.index_view.request.GET = {'sort_dir': 'asc', 'marker': 'foo'}
        self.assertTrue(self.index_view.has_prev_page())

    def test_paginate_queryset(self):
        result = self.index_view.paginate_queryset({}, 3)
        self.assertEqual((None, None, {}, None), result)

    def test_get_current_category(self):
        self.assertEqual('foo_category',
                         self.index_view.get_current_category())

    @mock.patch.object(views, 'reverse')
    def test_current_page_url(self, mock_reverse):
        mock_reverse.return_value = 'foo_curr_url'
        self.index_view.get_marker = lambda: 'foo_marker'

        result = self.index_view.current_page_url()
        result_parts = urlparse(result).query.split('&')
        expected = "sort_dir=desc&marker=foo_marker&search="\
                   "foo_search&order_by=foo_col".split('&')
        self.assertEqual('foo_curr_url', urlparse(result).path)
        self.assertEqual(sorted(expected), sorted(result_parts))

    @mock.patch.object(views, 'reverse')
    def test_prev_page_url(self, mock_reverse):
        mock_reverse.return_value = 'foo_prev_url'
        self.index_view.get_marker = lambda i: 'foo_marker'

        result = self.index_view.prev_page_url()
        result_parts = urlparse(result).query.split('&')
        expected = "sort_dir=desc&marker=foo_marker&search="\
                   "foo_search&order_by=foo_col".split('&')
        self.assertEqual('foo_prev_url', urlparse(result).path)
        self.assertEqual(sorted(expected), sorted(result_parts))

    @mock.patch.object(views, 'reverse')
    def test_next_page_url(self, mock_reverse):
        mock_reverse.return_value = 'foo_next_url'
        self.index_view.get_marker = lambda: 'foo_marker'

        result = self.index_view.next_page_url()
        result_parts = urlparse(result).query.split('&')
        expected = "sort_dir=asc&marker=foo_marker&search="\
                   "foo_search&order_by=foo_col".split('&')
        self.assertEqual('foo_next_url', urlparse(result).path)
        self.assertEqual(sorted(expected), sorted(result_parts))

    @mock.patch.object(views, 'reverse')
    @mock.patch.object(views, 'get_environments_context')
    @mock.patch.object(views, 'cleaned_latest_apps')
    @mock.patch.object(views, 'get_categories_list')
    def test_get_context_data(self, mock_get_categories_list,
                              mock_cleaned_latest_apps,
                              mock_get_environments_context, mock_reverse):
        mock_get_categories_list.return_value = [
            'foo_category', 'bar_category'
        ]
        mock_cleaned_latest_apps.return_value = ['foo_app', 'bar_app']
        mock_get_environments_context.return_value = {}
        mock_reverse.return_value = 'foo_url'
        mock_token = mock.Mock(tenant={'id': 'foo_tenant_id'})

        setattr(settings, 'MURANO_USE_GLARE', True)

        self.index_view.request.session = {'token': mock_token}
        self.index_view.object_list = []
        context_data = self.index_view.get_context_data()

        expected = {
            'ALL_CATEGORY_NAME': 'All',
            'MURANO_USE_GLARE': True,
            'categories': ['foo_category', 'bar_category'],
            'current_category': 'foo_category',
            'display_repo_url': 'http://apps.openstack.org/#tab=murano-apps',
            'is_paginated': None,
            'latest_list': ['foo_app', 'bar_app'],
            'no_apps': False,
            'object_list': [],
            'page_obj': None,
            'paginator': None,
            'pkg_def_url': 'foo_url',
            'search': 'foo_search',
            'tenant_id': 'foo_tenant_id',
            'view': self.index_view
        }

        for key, val in expected.items():
            self.assertEqual(val, context_data[key])
        mock_reverse.assert_called_once_with(
            'horizon:app-catalog:packages:index')


class TestAppDetailsView(unittest.TestCase):

    def setUp(self):
        super(TestAppDetailsView, self).setUp()
        self.app_details_view = views.AppDetailsView()
        self.app_details_view.request = mock.Mock(GET={})
        self.app_details_view.request.user.service_catalog = ['foo_service']

        self.assertEqual(catalog_tabs.ApplicationTabs,
                         self.app_details_view.tab_group_class)
        self.assertEqual('catalog/app_details.html',
                         self.app_details_view.template_name)
        self.assertEqual('{{ app.name }}', self.app_details_view.page_title)
        self.assertIsNone(self.app_details_view.app)

    @mock.patch.object(views, 'api')
    def test_get_data(self, mock_api):
        mock_api.muranoclient().packages.get.return_value = 'foo_app'
        kwargs = {'application_id': 'foo_app_id'}

        app = self.app_details_view.get_data(**kwargs)

        self.assertEqual('foo_app', app)
        self.assertEqual('foo_app', self.app_details_view.app)
        mock_api.muranoclient().packages.get.assert_called_with(
            'foo_app_id')

    @mock.patch.object(views, 'api')
    @mock.patch.object(views, 'get_environments_context')
    def test_get_context_data(self, mock_get_environments_context, mock_api):
        mock_api.muranoclient().packages.get.return_value = 'foo_app'
        mock_get_environments_context.return_value = {}

        context = self.app_details_view.get_context_data()
        expected = {'app': 'foo_app'}
        for key, val in expected.items():
            self.assertEqual(val, context[key])

    @mock.patch.object(views, 'api')
    def test_get_tabs(self, mock_api):
        mock_api.muranoclient().packages.get.return_value = 'foo_app'
        kwargs = {'application_id': 'foo_app_id'}

        tabs = self.app_details_view.get_tabs(self.app_details_view.request,
                                              **kwargs)
        self.assertIsInstance(tabs, catalog_tabs.ApplicationTabs)
        expected_kwargs = {
            'application': 'foo_app', 'application_id': 'foo_app_id'}
        for key, val in expected_kwargs.items():
            self.assertEqual(val, tabs.kwargs[key])



#    Copyright (c) 2016 AT&T
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import unittest

from muranodashboard.dynamic_ui import yaql_expression


class TestYaqlExpression(unittest.TestCase):
    def setUp(self):
        super(TestYaqlExpression, self).setUp()

        yaql = "$foo"
        string = "test"
        self.yaql_expr = yaql_expression.YaqlExpression(yaql)
        self.str_expr = yaql_expression.YaqlExpression(string)

    def test_overloading(self):
        self.assertEqual("test", self.str_expr.__str__())
        self.assertEqual("$foo", self.yaql_expr.__str__())

        self.assertEqual("YAQL(test)", self.str_expr.__repr__())
        self.assertEqual("YAQL($foo)", self.yaql_expr.__repr__())

    def test_expression(self):
        self.assertEqual("$foo", self.yaql_expr.expression())
        self.assertEqual("test", self.str_expr.expression())

    def test_match(self):
        self.assertFalse(self.str_expr.match(12345))
        self.assertFalse(self.str_expr.match(self.str_expr._expression))
        self.assertTrue(self.yaql_expr.match(self.yaql_expr._expression))
        self.assertFalse(self.yaql_expr.match("$!"))  # YaqlLexicalException
        self.assertFalse(self.yaql_expr.match("$foo("))  # YaqlGrammarException

    def test_evaluate(self):
        self.assertEqual("test", self.str_expr.evaluate())
        self.assertIsNone(self.yaql_expr.evaluate())



#    Copyright (c) 2015 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django import forms
from django.utils.translation import gettext_lazy as _
from horizon import forms as horizon_forms
from horizon import messages

from muranodashboard import api


class AddCategoryForm(horizon_forms.SelfHandlingForm):

    name = forms.CharField(label=_('Category Name'),
                           max_length=80,
                           help_text=_('80 characters max.'))

    def handle(self, request, data):
        if data:
            with api.handled_exceptions(self.request):
                category = api.muranoclient(self.request).categories.add(data)
                messages.success(request, _('Category {0} created.')
                                 .format(data['name']))
                return category
